##############################################################################
# Module : abcoption.py
# Author :
# Date   :
# Updated and modified for ABC_OKC : Old King Cole
#
# Description : ABC_OKC preferences dialog
#
##############################################################################
import sys, wx, win32con
from time import clock, time
from os import path, listdir
from binascii import hexlify
from string import whitespace

import wx.lib.masked as masked
from win32api import RegCreateKey, RegSetValueEx, RegCloseKey

from BitTornado.natpunch import setPeerID
from BitTornado.BTcrypto import CRYPTO_OK
from BitTornado.__init__ import setUserAgent, resetPeerID, resetUserPeerID, createPeerID, updateIdRandoms, getIdRandoms

from filemanager import *
from utility import SpinBox

sysencoding = sys.getfilesystemencoding()


class ABCOptionDialog(wx.Dialog):
    def __init__(self, parent, ID, title,
                 pos = wx.DefaultPosition, size = wx.DefaultSize,
                 style = wx.DEFAULT_DIALOG_STYLE):
        pre = wx.PreDialog()
        pre.Create(parent, ID, title, pos, size, style)
        self.this = pre.this
        self.parent = parent
        self.utility = self.parent.utility
        self.localize = self.utility.lang.get
        self.queue = self.parent.window.queue
        self.list = self.parent.window.list
        self.abcparams = self.utility.abcparams
        self.numsim = 0
        self.advancedMenuBox = None
        self.listfontface = self.abcparams['listfont']
        self.listfontsize = int(self.abcparams['listfontsize'])
        self.listfontstyle = int(self.abcparams['listfontstyle'])
        self.listfontweight = int(self.abcparams['listfontweight'])

        self.notebook = wx.Notebook(self, -1)
        self.notebook.SetPadding((4, 3))
        self.notebook.AddPage(self.makeLimitsPanel(self.notebook), self.localize('limitssetting'))
        self.notebook.AddPage(self.makeQueuePanel(self.notebook), self.localize('queuesetting'))
        self.notebook.AddPage(self.makeURMActivityPanel(self.notebook), self.localize('urmsetting'))
        self.notebook.AddPage(self.makeNetworkPanel(self.notebook), self.localize('networksetting'))
        self.notebook.AddPage(self.makeListPanel(self.notebook), self.localize('listsetting'))
        self.notebook.AddPage(self.makeInterfacePanel(self.notebook), self.localize('interfacesetting'))
        self.notebook.AddPage(self.makeMiscPanel(self.notebook), self.localize('miscsetting'))
        self.notebook.SetAutoLayout(True)

        advbtn = wx.Button(self, -1, self.localize('advsetting'))
        applybtn = wx.Button(self, -1, self.localize('apply'), size = (60, -1))
        okbtn = wx.Button(self, -1, self.localize('ok'), size = (60, -1))
        cancelbtn = wx.Button(self, -1, self.localize('cancel'), size = (60, -1))

        buttonbox = wx.BoxSizer(wx.HORIZONTAL)
        buttonbox.Add(advbtn, 0, wx.ALL, 5)
        buttonbox.Add(applybtn, 0, wx.ALL, 5)
        buttonbox.Add(okbtn, 0, wx.ALL, 5)
        buttonbox.Add(cancelbtn, 0, wx.ALL, 5)

        outerbox = wx.BoxSizer(wx.VERTICAL)
        outerbox.Add(self.notebook, 0, wx.LEFT | wx.RIGHT | wx.TOP, 5)
        outerbox.Add(buttonbox, 0, wx.ALIGN_CENTER)

        self.SetAutoLayout(True)
        self.SetSizer(outerbox)
        self.Fit()
        self.notebook.SetSelection(int(self.abcparams['lastopenabcoptab']))

        # Add events
        ###########################
        self.Bind(wx.EVT_BUTTON, self.onOK, okbtn)
        self.Bind(wx.EVT_BUTTON, self.onApply, applybtn)
        self.Bind(wx.EVT_BUTTON, self.onAdvSetting, advbtn)
        self.Bind(wx.EVT_BUTTON, self.onCancel, cancelbtn)
        self.Bind(wx.EVT_CLOSE, self.onCancel)
        self.Bind(wx.EVT_MOUSEWHEEL, self.onMouseWheel)

        if self.movedatastatus.GetSelection():
            self.completedresumedelaytext.Hide()
            self.completedresumedelay.Hide()
            self.completedresumedelayunit.Hide()
        else:
            self.completedresumedelaytext.Show()
            self.completedresumedelay.Show()
            self.completedresumedelayunit.Show()

        if self.abcparams['abcoptiondlgx'] != '-1':
            self.MoveXY(int(self.abcparams['abcoptiondlgx']), int(self.abcparams['abcoptiondlgy']))

    def getLanguages(self):
        langpath = path.join(self.utility.abcpath, "lang")
        dirlist = listdir(langpath)
        dirlist2 = []
        for filename in dirlist:
            if filename[-5:] == '.lang':
                dirlist2.append(filename)
        dirlist2.sort()

        langfiles = []
        langnames = []
        for filename in dirlist2:
            filepath = path.join(langpath, filename)
            f = open(filepath, "r")
            firstline = True
            while True:
                configline = f.readline()
                if configline == "":
                    break
                if firstline:
                    firstline = False
                    if configline[:3] == '\xef\xbb\xbf':
                        self.encoding = 'utf_8'
                        # Skip BOM
                        configline = configline[3:]
                        if configline == "":
                            break
                        if configline == "\n" or configline[0] == '#':
                            continue
                    else:
                        self.encoding = sysencoding
                if configline == "\n" or configline[0] == '#':
                    continue
                configmap = configline.decode(self.encoding).split("=", 1)
                if configmap[0] == 'languagename':
                    langfiles.append(filename)
                    langnames.append(configmap[1][1:-2])
                break
            f.close()

        return langfiles, langnames

    # Down and up Limits Setting
    ############################
    def makeLimitsPanel(self, parent):
        p = wx.Panel(parent, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)

        globalbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('globallimits')), wx.VERTICAL)

        self.numsimtext = SpinBox(p, header = self.localize('maxnumsimul'), value = self.abcparams['numsimdownload'])

        globalbox.Add(self.numsimtext.getSizer(), 0, wx.TOP, 0)

        downbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('downloadlimits')), wx.VERTICAL)

        maxoveralldownload = wx.BoxSizer(wx.HORIZONTAL)
        self.downloadrate = wx.TextCtrl(p, -1, self.abcparams['maxdownloadrate'], wx.Point(-1, -1), wx.Size(40, -1))        
        self.downloadrate.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        self.downloadrate.SetToolTipString(self.localize('0isnolimittt'))
        maxoveralldownload.Add(wx.StaticText(p, -1, self.localize('maxoverdownloadrate')), 0, wx.ALIGN_CENTER_VERTICAL)
        maxoveralldownload.Add(self.downloadrate, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        maxoveralldownload.Add(wx.StaticText(p, -1, self.utility.kbps), 0, wx.ALIGN_CENTER_VERTICAL)

        downbox.Add(maxoveralldownload, 0, wx.TOP, 0)

        downvolcheckbox = wx.BoxSizer(wx.HORIZONTAL)

        self.downvolcheck = wx.CheckBox(p, -1, self.localize('downvolcheck'))
        if self.abcparams['downvolcheck'] == "1":
            self.downvolcheck.SetValue(True)
        downvolcheckbox.Add(self.downvolcheck, 0, wx.ALIGN_CENTER_VERTICAL)

        self.downvolmax = wx.TextCtrl(p, -1, self.abcparams['downvolmax'], wx.Point(-1, -1), wx.Size(30, -1))
        self.downvolmax.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
        downvolcheckbox.Add(self.downvolmax, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 2)

        downvolcheckbox.Add(wx.StaticText(p, -1, self.localize('GB')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        downvolcheckbox.Add(wx.StaticText(p, -1,self.localize('over')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        self.downvolcountdur = wx.TextCtrl(p, -1, self.abcparams['downvolcountdur'], wx.Point(-1, -1), wx.Size(30, -1))
        self.downvolcountdur.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        downvolcheckbox.Add(self.downvolcountdur, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        downvolcheckbox.Add(wx.StaticText(p, -1, self.localize('hour_short')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        downvolcheckbox.Add(wx.StaticText(p, -1,self.localize('from')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        downvolnowbtn = wx.Button(p, -1, self.localize('now'), wx.Point(-1, -1), wx.Size(-1, 20), style = wx.BU_EXACTFIT)
        self.Bind(wx.EVT_BUTTON, self.onSetDownVolCheckNow, downvolnowbtn)

        downvolcheckbox.Add(downvolnowbtn, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        self.downvoldatebtn = wx.Button(p, -1, self.localize('date'), wx.Point(-1, -1), wx.Size(-1, 20), style = wx.BU_EXACTFIT)
        self.Bind(wx.EVT_BUTTON, self.onSetDownVolCheckDate, self.downvoldatebtn)

        downvolcheckbox.Add(self.downvoldatebtn, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        downbox.Add(downvolcheckbox, 0, wx.TOP, 10)

        upbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('uploadlimits')), wx.VERTICAL)

        statmaxuprate = wx.RadioButton(p, -1, self.localize('statmaxupratetxt'), style = wx.RB_GROUP)
        self.urmdynmaxuprate = wx.RadioButton(p, -1, self.localize('urmdynmaxupratetxt'))
        if self.abcparams['urmdynmaxuprate'] == "1":
            self.urmdynmaxuprate.SetValue(True)
        else:
            statmaxuprate.SetValue(True)

        upbox.Add(statmaxuprate, 0, wx.TOP, 0)

        maxoveralldown = wx.BoxSizer(wx.HORIZONTAL)
        self.uploadrate = wx.TextCtrl(p, -1, self.abcparams['maxuploadrate'], wx.Point(-1, -1), wx.Size(40, -1))        
        self.uploadrate.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        self.uploadrate.SetToolTipString(self.localize('0isnolimittt'))
        maxoveralldown.Add(wx.StaticText(p, -1, self.localize('whendownload')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 50)
        maxoveralldown.Add(self.uploadrate, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        maxoveralldown.Add(wx.StaticText(p, -1, self.utility.kbps), 0, wx.ALIGN_CENTER_VERTICAL)

        upbox.Add(maxoveralldown, 0, wx.TOP, 3)

        maxoverallnodown = wx.BoxSizer(wx.HORIZONTAL)
        self.seeduploadrate = wx.TextCtrl(p, -1, self.abcparams['maxseeduploadrate'], wx.Point(-1, -1), wx.Size(40, -1))
        self.seeduploadrate.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        self.seeduploadrate.SetToolTipString(self.localize('0isnolimittt'))
        maxoverallnodown.Add(wx.StaticText(p, -1, self.localize('whennodownload')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 50)
        maxoverallnodown.Add(self.seeduploadrate, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        maxoverallnodown.Add(wx.StaticText(p, -1, self.utility.kbps), 0, wx.ALIGN_CENTER_VERTICAL)

        upbox.Add(maxoverallnodown, 0, wx.TOP, 3)

        upbox.Add(self.urmdynmaxuprate, 0, wx.TOP, 3)

        urmmaxsysuprate = wx.BoxSizer(wx.HORIZONTAL)
        self.urmmaxsysupratetext = wx.TextCtrl(p, -1, self.abcparams['urmmaxsysuprate'], wx.Point(-1, -1), wx.Size(40, -1))
        self.urmmaxsysupratetext.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        urmmaxsysuprate.Add(wx.StaticText(p, -1, self.localize('urmmaxsysuprate')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 50)
        urmmaxsysuprate.Add(self.urmmaxsysupratetext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        urmmaxsysuprate.Add(wx.StaticText(p, -1, self.utility.kbps), 0, wx.ALIGN_CENTER_VERTICAL)

        upbox.Add(urmmaxsysuprate, 0, wx.TOP, 3)

        upvolcheckbox = wx.BoxSizer(wx.HORIZONTAL)

        self.upvolcheck = wx.CheckBox(p, -1, self.localize('upvolcheck'))
        if self.abcparams['upvolcheck'] == "1":
            self.upvolcheck.SetValue(True)
        upvolcheckbox.Add(self.upvolcheck, 0, wx.ALIGN_CENTER_VERTICAL)

        self.upvolmax = wx.TextCtrl(p, -1, self.abcparams['upvolmax'], wx.Point(-1, -1), wx.Size(30, -1))
        self.upvolmax.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
        upvolcheckbox.Add(self.upvolmax, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 2)

        upvolcheckbox.Add(wx.StaticText(p, -1, self.localize('GB')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        upvolcheckbox.Add(wx.StaticText(p, -1,self.localize('over')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        self.upvolcountdur = wx.TextCtrl(p, -1, self.abcparams['upvolcountdur'], wx.Point(-1, -1), wx.Size(30, -1))
        self.upvolcountdur.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        upvolcheckbox.Add(self.upvolcountdur, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        upvolcheckbox.Add(wx.StaticText(p, -1, self.localize('hour_short')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        upvolcheckbox.Add(wx.StaticText(p, -1,self.localize('from')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        upvolnowbtn = wx.Button(p, -1, self.localize('now'), wx.Point(-1, -1), wx.Size(-1, 20), style = wx.BU_EXACTFIT)
        self.Bind(wx.EVT_BUTTON, self.onSetUpVolCheckNow, upvolnowbtn)

        upvolcheckbox.Add(upvolnowbtn, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        self.upvoldatebtn = wx.Button(p, -1, self.localize('date'), wx.Point(-1, -1), wx.Size(-1, 20), style = wx.BU_EXACTFIT)
        self.Bind(wx.EVT_BUTTON, self.onSetUpVolCheckDate, self.upvoldatebtn)

        upvolcheckbox.Add(self.upvoldatebtn, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        upbox.Add(upvolcheckbox, 0, wx.TOP, 10)

        sizer.Add(globalbox, 0, wx.EXPAND | wx.TOP, 3)
        sizer.Add(downbox, 0, wx.EXPAND | wx.TOP, 3)
        sizer.Add(upbox, 0, wx.EXPAND | wx.TOP, 3)

        p.SetSizer(sizer)
        p.SetAutoLayout(True)

        return p

    # Queue Setting
    ##########################
    def makeQueuePanel(self, parent):
        p = wx.Panel(parent, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)

        queuetopbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('mainschedulertitle')), wx.VERTICAL)

        trig_finish_values = [self.localize('after_downloading'), self.localize('after_seeding')]
        self.trig_finish_seed = wx.ComboBox(p, -1, '', (-1, -1), (-1, -1), trig_finish_values,
                                            wx.CB_DROPDOWN | wx.CB_READONLY)
        self.trig_finish_seed.SetSelection(int(self.abcparams['trigwhenfinishseed']))

        trigger_box = wx.BoxSizer(wx.HORIZONTAL)
        trigger_box.Add(wx.StaticText(p, -1, self.localize('trignexttorrent')), 0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        trigger_box.Add(self.trig_finish_seed, 0, wx.ALIGN_CENTER_VERTICAL)

        queuetopbox.Add(trigger_box, 0, wx.TOP, 0)

        self.defaultpriority = wx.ComboBox(p, -1, '', (-1,-1), (-1, -1),
                                           [self.localize('highest'), self.localize('high'), self.localize('normal'), self.localize('low'), self.localize('lowest')],
                                           wx.CB_DROPDOWN | wx.CB_READONLY)
        self.defaultpriority.SetSelection(int(self.abcparams['defaultpriority']))

        prio_box = wx.BoxSizer(wx.HORIZONTAL)
        prio_box.Add(wx.StaticText(p, -1, self.localize('defaultpriority')), 0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        prio_box.Add(self.defaultpriority, 0, wx.ALIGN_CENTER_VERTICAL)       

        queuetopbox.Add(prio_box, 0, wx.TOP, 3)

        prioritizelocalbox = wx.BoxSizer(wx.HORIZONTAL)

        prioritizelocalbox.Add(wx.StaticText(p, -1, self.localize('prioritizelocal')), 0, wx.ALIGN_LEFT)

        self.prioritizelocaldown = wx.CheckBox(p, -1, self.localize('prioritizelocaldown'))
        if self.abcparams['prioritizelocaldown'] == "1":
            self.prioritizelocaldown.SetValue(True)
        prioritizelocalbox.Add(self.prioritizelocaldown, 0, wx.ALIGN_LEFT | wx.LEFT, 10)

        self.prioritizelocal = wx.CheckBox(p, -1, self.localize('prioritizelocalup'))
        if self.abcparams['prioritizelocal'] == "1":
            self.prioritizelocal.SetValue(True)
        prioritizelocalbox.Add(self.prioritizelocal, 0, wx.ALIGN_LEFT | wx.LEFT, 15)

        queuetopbox.Add(prioritizelocalbox, 0, wx.TOP, 3)

        fastresumebox = wx.BoxSizer(wx.HORIZONTAL)

        fastresumebox.Add(wx.StaticText(p, -1, self.localize('fastresumetext')), 0, wx.ALIGN_LEFT)

        self.fastresume = wx.CheckBox(p, -1, self.localize('fastresume'))
        if self.abcparams['fastresume'] == "1":
            self.fastresume.SetValue(True)
        fastresumebox.Add(self.fastresume, 0, wx.ALIGN_LEFT | wx.LEFT, 10)

        self.fastresumefirst = wx.CheckBox(p, -1, self.localize('fastresumefirst'))
        if self.abcparams['fastresumefirst'] == "1":
            self.fastresumefirst.SetValue(True)
        fastresumebox.Add(self.fastresumefirst, 0, wx.ALIGN_LEFT | wx.LEFT, 15)

        queuetopbox.Add(fastresumebox, 0, wx.TOP, 3)

        self.sbqueued = wx.CheckBox(p, -1, self.localize('sbqueued'))
        if self.abcparams['sbqueued'] == "1":
            self.sbqueued.SetValue(True)

        queuetopbox.Add(self.sbqueued, 0, wx.TOP, 3)

        self.schedonhold = wx.CheckBox(p, -1, self.localize('schedonhold'))
        if self.abcparams['schedonhold'] == "1":
            self.schedonhold.SetValue(True)

        queuetopbox.Add(self.schedonhold, 0, wx.TOP, 3)

        self.schedstopsman = wx.CheckBox(p, -1, self.localize('schedstopsman'))
        if self.abcparams['schedstopsman'] == "1":
            self.schedstopsman.SetValue(True)

        queuetopbox.Add(self.schedstopsman, 0, wx.TOP, 3)

        self.scheddonthalt = wx.CheckBox(p, -1, self.localize('scheddonthalt'))
        if self.abcparams['scheddonthalt'] == "1":
            self.scheddonthalt.SetValue(True)
        self.scheddonthaltuprategap = wx.TextCtrl(p, -1, self.abcparams['scheddonthaltuprategap'], wx.Point(-1, -1), wx.Size(40, -1))        
        self.scheddonthaltuprategap.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        scheddonthaltbox = wx.BoxSizer(wx.HORIZONTAL)
        scheddonthaltbox.Add(self.scheddonthalt, 0, wx.ALIGN_CENTER_VERTICAL)
        scheddonthaltbox.Add(self.scheddonthaltuprategap, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 2)
        scheddonthaltbox.Add(wx.StaticText(p, -1, self.utility.kbps), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        queuetopbox.Add(scheddonthaltbox, 0, wx.TOP, 3)

        self.lockedcompletedcheck = wx.CheckBox(p, -1, self.localize('lockedcompletedcheck'))
        if self.abcparams['lockedcompletedcheck'] == "1":
            self.lockedcompletedcheck.SetValue(True)
        self.lockedcompletedmaxtext = SpinBox(p, value = self.abcparams['lockedcompletedmax'])
        lockedcompletedbox = wx.BoxSizer(wx.HORIZONTAL)
        lockedcompletedbox.Add(self.lockedcompletedcheck, 0, wx.ALIGN_CENTER_VERTICAL)
        lockedcompletedbox.Add(self.lockedcompletedmaxtext.getSizer(), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 2)
        lockedcompletedbox.Add(wx.StaticText(p, -1, self.localize('lockedcompletedmax')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        queuetopbox.Add(lockedcompletedbox, 0, wx.TOP, 3)

        self.stoprejected = wx.CheckBox(p, -1, self.localize('stoprejected'))
        if self.abcparams['stoprejected'] == "1":
            self.stoprejected.SetValue(True)

        queuetopbox.Add(self.stoprejected, 0, wx.TOP, 3)

        rejectexceptionsbox = wx.BoxSizer(wx.HORIZONTAL)
        rejectexceptionsbox.Add(wx.StaticText(p, -1, self.localize('rejectexceptions')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 20)
        self.rejectexceptions = wx.TextCtrl(p, -1, self.abcparams['rejectexceptions'])
        self.rejectexceptions.Bind(wx.EVT_CHAR, self.utility.onWriteFilteredText)
        rejectexceptionsbox.Add(self.rejectexceptions, 1, wx.ALIGN_CENTER_VERTICAL | wx.EXPAND | wx.LEFT, 5)

        queuetopbox.Add(rejectexceptionsbox, 0,  wx.EXPAND | wx.TOP, 3)

        sizer.Add(queuetopbox, 0, wx.EXPAND | wx.TOP, 3)

        torrenttimeoutbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('globaltorrentimeout')), wx.VERTICAL)

        self.timeoutwhenextra = wx.CheckBox(p, -1, self.localize('timeoutwhenextra'))
        if self.abcparams['timeoutwhenextra'] == "1":
            self.timeoutwhenextra.SetValue(True)

        torrenttimeoutbox.Add(self.timeoutwhenextra, 0, wx.TOP, 3)

        canceltimeoutsbox = wx.BoxSizer(wx.HORIZONTAL)
        self.canceltimeouts = wx.CheckBox(p, -1, self.localize('canceltimeouts'))
        if self.abcparams['canceltimeouts'] == "1":
            self.canceltimeouts.SetValue(True)
        canceltimeoutsbox.Add(self.canceltimeouts, 0, wx.ALIGN_CENTER_VERTICAL)

        self.canceltimeoutsmsg = wx.TextCtrl(p, -1, self.abcparams['canceltimeoutsmsg'])
        self.canceltimeoutsmsg.Bind(wx.EVT_CHAR, self.utility.onWriteFilteredText)
        canceltimeoutsbox.Add(self.canceltimeoutsmsg, 1, wx.ALIGN_CENTER_VERTICAL | wx.EXPAND | wx.LEFT, 2)

        torrenttimeoutbox.Add(canceltimeoutsbox, 0, wx.EXPAND | wx.TOP, 3)

        sbtimeoutbox = wx.BoxSizer(wx.HORIZONTAL)
        self.sbtimeout = wx.CheckBox(p, -1, self.localize('sbtimeout'))
        if self.abcparams['sbtimeout'] == "1":
            self.sbtimeout.SetValue(True)
        sbtimeoutbox.Add(self.sbtimeout, 0, wx.ALIGN_CENTER_VERTICAL)
        self.sbtimeoutvalue = wx.TextCtrl(p, -1, self.abcparams['sbtimeoutvalue'], wx.Point(-1, -1), wx.Size(30, -1))
        self.sbtimeoutvalue.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        sbtimeoutbox.Add(self.sbtimeoutvalue, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 2)
        sbtimeoutbox.Add(wx.StaticText(p, -1, self.localize('sbtimeoutvalue')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        sbtimeoutbox.Add(wx.StaticText(p, -1, self.localize('sbtimeoutaction')), 0, wx.ALIGN_CENTER_VERTICAL)
        self.sbtimeoutaction = wx.ComboBox(p, -1, '', (-1, -1), (-1, -1),
                                           [self.localize('sbtimeoutactionqueue'), self.localize('sbtimeoutactionstop')],
                                           wx.CB_DROPDOWN | wx.CB_READONLY)
        self.sbtimeoutaction.SetSelection(int(self.abcparams['sbtimeoutaction']))
        sbtimeoutbox.Add(self.sbtimeoutaction, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        torrenttimeoutbox.Add(sbtimeoutbox, 0, wx.TOP, 3)
 
        sizer.Add(torrenttimeoutbox, 0, wx.EXPAND | wx.TOP, 3)

        automovesbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('automoves')), wx.VERTICAL)

        self.torrentwhencompleted = wx.ComboBox(p, -1, '', (-1, -1), (-1, -1),
                                                [self.localize('donothing'), self.localize('movetotop')],
                                                wx.CB_DROPDOWN | wx.CB_READONLY)
        self.torrentwhencompleted.SetSelection(int(self.abcparams['torrentwhencompleted']))
        twcbox = wx.BoxSizer(wx.HORIZONTAL)
        twcbox.Add(wx.StaticText(p, -1, self.localize('torrentcompleted')), 0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        twcbox.Add(self.torrentwhencompleted, 0, wx.ALIGN_CENTER_VERTICAL)       

        automovesbox.Add(twcbox, 0, wx.TOP, 0)

        self.torrentwhenfinished = wx.ComboBox(p, -1, '', (-1, -1), (-1, -1),
                                               [self.localize('donothing'), self.localize('movetotop'), self.localize('deletetorrent')],
                                               wx.CB_DROPDOWN | wx.CB_READONLY)
        self.torrentwhenfinished.SetSelection(int(self.abcparams['torrentwhenfinished']))
        twfbox = wx.BoxSizer(wx.HORIZONTAL)
        twfbox.Add(wx.StaticText(p, -1, self.localize('torrentfinished')), 0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        twfbox.Add(self.torrentwhenfinished, 0, wx.ALIGN_CENTER_VERTICAL)       

        automovesbox.Add(twfbox, 0, wx.TOP, 3)

        self.sortbytime = wx.CheckBox(p, -1, self.localize('sortbytime'))
        if self.abcparams['sortbytime'] == "1":
            self.sortbytime.SetValue(True)

        automovesbox.Add(self.sortbytime, 0, wx.TOP, 3)

        self.sorttoronclose = wx.CheckBox(p, -1, self.localize('sorttoronclose'))
        if self.abcparams['sorttoronclose'] == "1":
            self.sorttoronclose.SetValue(True)

        automovesbox.Add(self.sorttoronclose, 0, wx.TOP, 3)

        sizer.Add(automovesbox, 0, wx.EXPAND | wx.TOP, 3)

        p.SetSizer(sizer)
        p.SetAutoLayout(True)
        return p

    # URM and activity Setting
    ##########################
    def makeURMActivityPanel(self, parent):
        p = wx.Panel(parent, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)

        urmschedulerbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('urmschedulertitle')), wx.VERTICAL)

        self.urm = wx.CheckBox(p, -1, self.localize('urm'))
        if self.abcparams['urm'] == "1":
            self.urm.SetValue(True)
        urmschedulerbox.Add(self.urm, 0, wx.TOP, 0)

        self.urmonhold = wx.CheckBox(p, -1, self.localize('urmonhold'))
        if self.abcparams['urmonhold'] == "1":
            self.urmonhold.SetValue(True)
        urmschedulerbox.Add(self.urmonhold, 0, wx.TOP, 3)

        self.urmstopsman = wx.CheckBox(p, -1, self.localize('urmstopsman'))
        if self.abcparams['urmstopsman'] == "1":
            self.urmstopsman.SetValue(True)
        urmschedulerbox.Add(self.urmstopsman, 0, wx.TOP, 3)

        self.urmstopsonlyactive = wx.CheckBox(p, -1, self.localize('urmstopsonlyactive'))
        if self.abcparams['urmstopsonlyactive'] == "1":
            self.urmstopsonlyactive.SetValue(True)
        urmschedulerbox.Add(self.urmstopsonlyactive, 0, wx.TOP, 3)

        self.urmstopsonlylowprio = wx.CheckBox(p, -1, self.localize('urmstopsonlylowprio'))
        if self.abcparams['urmstopsonlylowprio'] == "1":
            self.urmstopsonlylowprio.SetValue(True)
        urmschedulerbox.Add(self.urmstopsonlylowprio, 0, wx.TOP, 3)

        self.urmstoponpause = wx.CheckBox(p, -1, self.localize('urmstoponpause'))
        if self.abcparams['urmstoponpause'] == "1":
            self.urmstoponpause.SetValue(True)
        urmschedulerbox.Add(self.urmstoponpause, 0, wx.TOP, 3)

        self.urmmaxtorrenttext = SpinBox(p, header = self.localize('urmmaxtorrent'), value = self.abcparams['urmmaxtorrent'])

        self.urmlowpriority = wx.CheckBox(p, -1, self.localize('urmlowpriority'))
        if self.abcparams['urmlowpriority'] == "1":
            self.urmlowpriority.SetValue(True)
        urmbox1 = wx.BoxSizer(wx.HORIZONTAL)
        urmbox1.Add(self.urmmaxtorrenttext.getSizer(), 0, wx.ALIGN_CENTER_VERTICAL)
        urmbox1.Add(self.urmlowpriority, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 30)
        urmschedulerbox.Add(urmbox1, 0, wx.TOP, 3)

        self.urmthresholdstarttext = wx.TextCtrl(p, -1, self.abcparams['urmthresholdstart'], wx.Point(-1, -1), wx.Size(30, -1))
        self.urmthresholdstarttext.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
        self.urmthresholdstoptext = wx.TextCtrl(p, -1, self.abcparams['urmthresholdstop'], wx.Point(-1, -1), wx.Size(30, -1))
        self.urmthresholdstoptext.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
        urmthresholdstartbox = wx.BoxSizer(wx.HORIZONTAL)
        urmthresholdstartbox.Add(wx.StaticText(p, -1, self.localize('urmthresholdstart')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmthresholdstartbox.Add(self.urmthresholdstarttext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        urmthresholdstartbox.Add(wx.StaticText(p, -1, self.utility.kbps), 0, wx.ALIGN_CENTER_VERTICAL)
        urmthresholdstopbox = wx.BoxSizer(wx.HORIZONTAL)
        urmthresholdstopbox.Add(wx.StaticText(p, -1, self.localize('urmthresholdstop')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmthresholdstopbox.Add(self.urmthresholdstoptext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        urmthresholdstopbox.Add(wx.StaticText(p, -1, self.utility.kbps), 0, wx.ALIGN_CENTER_VERTICAL)
        urmbox2 = wx.BoxSizer(wx.HORIZONTAL)
        urmbox2.Add(wx.StaticText(p, -1, self.localize('urmthreshold')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmbox2.Add(urmthresholdstartbox, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 7)
        urmbox2.Add(urmthresholdstopbox, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 10)
        urmschedulerbox.Add(urmbox2, 0, wx.TOP, 3)

        self.urmdelaystarttext = wx.TextCtrl(p, -1, self.abcparams['urmdelay'], wx.Point(-1, -1), wx.Size(30, -1))
        self.urmdelaystarttext.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        self.urmdelaystoptext = wx.TextCtrl(p, -1, self.abcparams['urmdelaystop'], wx.Point(-1, -1), wx.Size(30, -1))
        self.urmdelaystoptext.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        urmdelaystartbox = wx.BoxSizer(wx.HORIZONTAL)
        urmdelaystartbox.Add(wx.StaticText(p, -1, self.localize('urmdelaystart')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmdelaystartbox.Add(self.urmdelaystarttext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        urmdelaystartbox.Add(wx.StaticText(p, -1, self.localize('l_second')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmdelaystopbox = wx.BoxSizer(wx.HORIZONTAL)
        urmdelaystopbox.Add(wx.StaticText(p, -1, self.localize('urmdelaystop')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmdelaystopbox.Add(self.urmdelaystoptext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        urmdelaystopbox.Add(wx.StaticText(p, -1, self.localize('l_second')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmbox3 = wx.BoxSizer(wx.HORIZONTAL)
        urmbox3.Add(wx.StaticText(p, -1, self.localize('urmdelay')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmbox3.Add(urmdelaystartbox, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 7)
        urmbox3.Add(urmdelaystopbox, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 10)
        urmschedulerbox.Add(urmbox3, 0, wx.TOP, 3)

        self.urmtorrentstartdelaytext = wx.TextCtrl(p, -1, self.abcparams['urmtorrentstartdelay'], wx.Point(-1, -1), wx.Size(30, -1))
        self.urmtorrentstartdelaytext.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        self.urmtorrentstopdelaytext = wx.TextCtrl(p, -1, self.abcparams['urmtorrentstopdelay'], wx.Point(-1, -1), wx.Size(30, -1))
        self.urmtorrentstopdelaytext.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        urmtorrentstartdelaybox = wx.BoxSizer(wx.HORIZONTAL)
        urmtorrentstartdelaybox.Add(wx.StaticText(p, -1, self.localize('urmtorrentstartdelay')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmtorrentstartdelaybox.Add(self.urmtorrentstartdelaytext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        urmtorrentstartdelaybox.Add(wx.StaticText(p, -1, self.localize('l_second')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmtorrentstopdelaybox = wx.BoxSizer(wx.HORIZONTAL)
        urmtorrentstopdelaybox.Add(wx.StaticText(p, -1, self.localize('urmtorrentstopdelay')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmtorrentstopdelaybox.Add(self.urmtorrentstopdelaytext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        urmtorrentstopdelaybox.Add(wx.StaticText(p, -1, self.localize('l_second')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmbox4 = wx.BoxSizer(wx.HORIZONTAL)
        urmbox4.Add(wx.StaticText(p, -1, self.localize('urmmindelay')), 0, wx.ALIGN_CENTER_VERTICAL)
        urmbox4.Add(urmtorrentstartdelaybox, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 7)
        urmbox4.Add(urmtorrentstopdelaybox, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 10)
        urmschedulerbox.Add(urmbox4, 0, wx.TOP, 3)

        sizer.Add(urmschedulerbox, 0, wx.EXPAND | wx.TOP, 3)

        if self.utility.tweakdynmaxuprate == 1:
            tweakdynmaxupratebox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('urmdynmaxupratetitle')), wx.VERTICAL)
    
            self.urmupfromdownAtext = wx.TextCtrl(p, -1, str(self.utility.urmupfromdownA), wx.Point(-1, -1), wx.Size(45, -1))
            self.urmupfromdownAtext.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
            self.urmupfromdownBtext = wx.TextCtrl(p, -1, str(self.utility.urmupfromdownB), wx.Point(-1, -1), wx.Size(45, -1))
            self.urmupfromdownBtext.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
            self.urmupfromdownCtext = wx.TextCtrl(p, -1, str(self.utility.urmupfromdownC), wx.Point(-1, -1), wx.Size(45, -1))
            self.urmupfromdownCtext.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
            self.urmupfromdownDtext = wx.TextCtrl(p, -1, str(self.utility.urmupfromdownD), wx.Point(-1, -1), wx.Size(45, -1))
            self.urmupfromdownDtext.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
            self.urmupfromdownEtext = wx.TextCtrl(p, -1, str(self.utility.urmupfromdownE), wx.Point(-1, -1), wx.Size(45, -1))
            self.urmupfromdownEtext.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
            urmupfromdownbox1 = wx.BoxSizer(wx.HORIZONTAL)
            urmupfromdownbox1.Add(wx.StaticText(p, -1, self.localize('urmupfromdown1')), 0, wx.ALIGN_CENTER_VERTICAL)
            urmupfromdownbox1.Add(self.urmupfromdownAtext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            urmupfromdownbox1.Add(self.urmupfromdownBtext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            urmupfromdownbox1.Add(self.urmupfromdownCtext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            urmupfromdownbox1.Add(self.urmupfromdownDtext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            urmupfromdownbox1.Add(self.urmupfromdownEtext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            tweakdynmaxupratebox.Add(urmupfromdownbox1, 0, wx.TOP, 0)

            self.urmupfromdownFtext = wx.TextCtrl(p, -1, str(self.utility.urmupfromdownF), wx.Point(-1, -1), wx.Size(45, -1))
            self.urmupfromdownFtext.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
            self.urmupfromdownGtext = wx.TextCtrl(p, -1, str(self.utility.urmupfromdownG), wx.Point(-1, -1), wx.Size(45, -1))
            self.urmupfromdownGtext.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
            urmupfromdownbox2 = wx.BoxSizer(wx.HORIZONTAL)
            urmupfromdownbox2.Add(wx.StaticText(p, -1, self.localize('urmupfromdown2')), 0, wx.ALIGN_CENTER_VERTICAL)
            urmupfromdownbox2.Add(self.urmupfromdownFtext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            urmupfromdownbox2.Add(self.urmupfromdownGtext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            tweakdynmaxupratebox.Add(urmupfromdownbox2, 0, wx.TOP, 3)

            sizer.Add(tweakdynmaxupratebox, 0, wx.EXPAND | wx.TOP, 3)

        activitybox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('activitytitle')), wx.VERTICAL)

        activitythbox = wx.BoxSizer(wx.HORIZONTAL)

        activitythbox.Add(wx.StaticText(p, -1, self.localize('activityth')), 0, wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT)
        activitythbox.Add(wx.StaticText(p, -1, self.localize('activitythdow')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 7)
        self.activitythdown = wx.TextCtrl(p, -1, self.abcparams['activitythdown'], wx.Point(-1, -1), wx.Size(30, -1))
        self.activitythdown.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
        activitythbox.Add(self.activitythdown, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        activitythbox.Add(wx.StaticText(p, -1, self.utility.kbps), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        activitythbox.Add(wx.StaticText(p, -1, self.localize('activitythup')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 10)
        self.activitythup = wx.TextCtrl(p, -1, self.abcparams['activitythup'], wx.Point(-1, -1), wx.Size(30, -1))
        self.activitythup.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDot)
        activitythbox.Add(self.activitythup, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        activitythbox.Add(wx.StaticText(p, -1, self.utility.kbps), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        activitythbox.Add(wx.StaticText(p, -1, self.localize('activitydelay')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 15)
        self.activitydelay = wx.TextCtrl(p, -1, self.abcparams['activitydelay'], wx.Point(-1, -1), wx.Size(30, -1))
        self.activitydelay.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        activitythbox.Add(self.activitydelay, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        activitythbox.Add(wx.StaticText(p, -1, self.localize('l_second')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        activitybox.Add(activitythbox, 0, wx.TOP, 0)

        self.activityextra = wx.CheckBox(p, -1, self.localize('activityextra'))
        if self.abcparams['activityextra'] == "1":
            self.activityextra.SetValue(True)
        activitybox.Add(self.activityextra, 0, wx.TOP, 3)

        self.activitymaxtext = SpinBox(p, header = self.localize('activitymax'), value = self.abcparams['activitymax'])

        activitybox.Add(self.activitymaxtext.getSizer(), 0, wx.TOP, 3)

        extratobestoppedbox = wx.BoxSizer(wx.HORIZONTAL)

        extratobestoppedbox.Add(wx.StaticText(p, -1, self.localize('extratobestopped')), 0, wx.ALIGN_LEFT)
        extratobestopped = int(self.abcparams['extratobestopped'])
        self.extratobestoppedactive = wx.CheckBox(p, -1, self.localize('extratobestoppedactive'),
                                                  style = wx.CHK_3STATE | wx.CHK_ALLOW_3RD_STATE_FOR_USER)
        self.extratobestoppedactive.SetToolTipString(self.localize('extratobestoppedactivett'))
        self.extratobestoppedactive.Set3StateValue(extratobestopped % 10)
        extratobestoppedbox.Add(self.extratobestoppedactive, 0, wx.ALIGN_LEFT | wx.LEFT, 10)
        self.extratobestoppedinactive = wx.CheckBox(p, -1, self.localize('extratobestoppedinactive'))
        if extratobestopped / 10:
            self.extratobestoppedinactive.SetValue(True)
        extratobestoppedbox.Add(self.extratobestoppedinactive, 0, wx.ALIGN_LEFT | wx.LEFT, 15)

        activitybox.Add(extratobestoppedbox, 0, wx.TOP, 3)

        self.dontcheckseedactivity = wx.CheckBox(p, -1, self.localize('dontcheckseedactivity'))
        if self.abcparams['dontcheckseedactivity'] == "1":
            self.dontcheckseedactivity.SetValue(True)

        activitybox.Add(self.dontcheckseedactivity, 0, wx.TOP, 3)

        sizer.Add(activitybox, 0, wx.EXPAND | wx.TOP, 3)

        p.SetSizer(sizer)
        p.SetAutoLayout(True)
        return p

    # Network Setting
    ####################
    def makeNetworkPanel(self, parent):
        p = wx.Panel(parent, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)

        portbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('porttitle')), wx.VERTICAL)

        portsizer = wx.BoxSizer(wx.HORIZONTAL)
        self.minport = wx.TextCtrl(p, -1, self.abcparams['minport'], wx.Point(-1, -1), wx.Size(45, -1))
        self.minport.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        minport_box = wx.BoxSizer(wx.HORIZONTAL)
        minport_box.Add(wx.StaticText(p, -1, self.localize('minportnumber')), 0, wx.ALIGN_CENTER_VERTICAL)
        minport_box.Add(self.minport, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        portsizer.Add(minport_box)

        self.maxport = wx.TextCtrl(p, -1, self.abcparams['maxport'], wx.Point(-1, -1), wx.Size(45, -1))
        self.maxport.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        maxport_box = wx.BoxSizer(wx.HORIZONTAL)
        maxport_box.Add(wx.StaticText(p, -1, self.localize('maxportnumber')), 0, wx.ALIGN_CENTER_VERTICAL)
        maxport_box.Add(self.maxport, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        portsizer.Add(maxport_box, 0, wx.LEFT, 50)

        portbox.Add(portsizer, 0, wx.TOP, 0)

        sizer.Add(portbox, 0, wx.EXPAND | wx.TOP, 3)

        cryptobox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('cryptotitle')), wx.VERTICAL)

        cryptorb0 = wx.RadioButton(p, -1, self.localize('cryptooff'), (-1, -1), (-1, -1), wx.RB_GROUP)
        cryptorb1 = wx.RadioButton(p, -1, self.localize('cryptoon'), (-1, -1), (-1, -1))
        cryptorb2 = wx.RadioButton(p, -1, self.localize('cryptoonly'), (-1, -1), (-1, -1))
        cryptorb3 = wx.RadioButton(p, -1, self.localize('cryptostealth'), (-1, -1), (-1, -1))
        self.crypto = [cryptorb0, cryptorb1, cryptorb2, cryptorb3]

        cryptobox.Add(cryptorb0, 0, wx.TOP, 0)
        cryptobox.Add(cryptorb1, 0, wx.TOP, 3)
        cryptobox.Add(cryptorb2, 0, wx.TOP, 3)
        cryptobox.Add(cryptorb3, 0, wx.TOP, 3)

        sizer.Add(cryptobox, 0, wx.EXPAND | wx.TOP, 3)

        if not CRYPTO_OK:   # No crypto library in place
            cryptorb0.Enable(False)
            cryptorb1.Enable(False)
            cryptorb2.Enable(False)
            cryptorb3.Enable(False)
        else:
            self.crypto[int(self.abcparams['crypto'])].SetValue(True)

        useragentidbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('useragentid')), wx.VERTICAL)

        useragentidrb0 = wx.RadioButton(p, -1, self.localize('default'), style = wx.RB_GROUP)
        useragentidrb1 = wx.RadioButton(p, -1, self.localize('user1'))
        useragentidrb2 = wx.RadioButton(p, -1, self.localize('user2'))
        useragentidrb3 = wx.RadioButton(p, -1, self.localize('user3'))
        self.useragentid = [useragentidrb0, useragentidrb1, useragentidrb2, useragentidrb3]

        uai0box = wx.BoxSizer(wx.HORIZONTAL)
        uai0box.Add(useragentidrb0, 0)
        uai0box.Add(wx.StaticText(p, -1, '[' + self.parent.defuseragent + '] / [ABC_OKC ID]'), 0, wx.LEFT, 2)
        useragentidbox.Add(uai0box, 0, wx.TOP, 0)

        self.useragentctrl = [None]
        self.useridctrl = [None]
        self.useridmaskbtn = [None]
        self.userid = [None]
        self.useridmask = [None]
        self._idrandoms = getIdRandoms()
        useragentidrb0.Bind(wx.EVT_RADIOBUTTON, self.onUserAgentIdRadioButton)
        for i in xrange(1, 4):
            self.useragentctrl.append(wx.TextCtrl(p, -1, self.abcparams['useragent' + str(i)], size = (85, -1)))
            self.useragentctrl[i].Bind(wx.EVT_CHAR, self.utility.onWriteFilteredText)
            self.userid.append(self.abcparams['userid' + str(i)])
            self.useridctrl.append(wx.TextCtrl(p, -1, self.userid[i], size = (172, -1)))
            self.useridctrl[i].Bind(wx.EVT_CHAR, self.utility.onWritePrintable)
            self.useridctrl[i].Bind(wx.EVT_TEXT, self.onWriteInUserPeerId)
            self.useridmask.append(self.abcparams['useridmask' + str(i)])
            self.useridmaskbtn.append(wx.ToggleButton(p, -1, self.localize('useridmaskbtn'), size = (18, -1)))
            if self.useridmask[i]:
                self.useridmaskbtn[i].SetLabel(self.localize('useridmaskbtn2'))
            self.Bind(wx.EVT_TOGGLEBUTTON, self.onUserPeerIdMaskToggle, self.useridmaskbtn[i])
            self.useragentid[i].Bind(wx.EVT_RADIOBUTTON, self.onUserAgentIdRadioButton)
        # 2 following to avoid computing ID when applying if neither ID nor mask were changed since last generation
        self.useridlastgenerate = self.userid[:]
        self.useridmasklastgenerate = self.useridmask[:]

        uai1box = wx.BoxSizer(wx.HORIZONTAL)
        uai1box.Add(useragentidrb1, 0, wx.ALIGN_CENTER_VERTICAL)
        uai1box.Add(self.useragentctrl[1], 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 2)
        uai1box.Add(wx.StaticText(p, -1, ' / '), 0, wx.ALIGN_CENTER_VERTICAL)
        uai1box.Add(self.useridctrl[1], 0, wx.ALIGN_CENTER_VERTICAL)
        uai1box.Add(self.useridmaskbtn[1], 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 1)
        useragentidbox.Add(uai1box, 0, wx.TOP, 3)

        uai2box = wx.BoxSizer(wx.HORIZONTAL)
        uai2box.Add(useragentidrb2, 0, wx.ALIGN_CENTER_VERTICAL)
        uai2box.Add(self.useragentctrl[2], 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 2)
        uai2box.Add(wx.StaticText(p, -1, ' / '), 0, wx.ALIGN_CENTER_VERTICAL)
        uai2box.Add(self.useridctrl[2], 0, wx.ALIGN_CENTER_VERTICAL)
        uai2box.Add(self.useridmaskbtn[2], 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 1)
        useragentidbox.Add(uai2box, 0, wx.TOP, 3)

        uai3box = wx.BoxSizer(wx.HORIZONTAL)
        uai3box.Add(useragentidrb3, 0, wx.ALIGN_CENTER_VERTICAL)
        uai3box.Add(self.useragentctrl[3], 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 2)
        uai3box.Add(wx.StaticText(p, -1, ' / '), 0, wx.ALIGN_CENTER_VERTICAL)
        uai3box.Add(self.useridctrl[3], 0, wx.ALIGN_CENTER_VERTICAL)
        uai3box.Add(self.useridmaskbtn[3], 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 1)
        useragentidbox.Add(uai3box, 0, wx.TOP, 3)

        self.useragentidnb = int(self.abcparams['useragentid'])
        self.useragentid[self.useragentidnb].SetValue(True)

        currentclientidbox = wx.BoxSizer(wx.HORIZONTAL)
        generateidbtn = wx.Button(p, -1, self.localize('generateidbtn'), size = (-1, 20), style = wx.BU_EXACTFIT)
        self.Bind(wx.EVT_BUTTON, self.onGenerateUserPeerId, generateidbtn)
        currentclientidbox.Add(generateidbtn, 0, wx.ALIGN_CENTER_VERTICAL)
        self.currentclientid = wx.TextCtrl(p, -1, self.binToString(createPeerID(useragentidnb = self.useragentidnb)), style = wx.TE_READONLY)
        currentclientidbox.Add(self.currentclientid, 1, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        useragentidbox.Add(currentclientidbox, 1, wx.EXPAND | wx.TOP, 3)

        sizer.Add(useragentidbox, 0, wx.EXPAND | wx.TOP, 3)

        networkbottombox = wx.StaticBoxSizer(wx.StaticBox(p, -1, ''), wx.VERTICAL)

        self.utpex = wx.CheckBox(p, -1, self.localize('utpex'))
        if self.abcparams['utpex'] == "1":
            self.utpex.SetValue(True)
        networkbottombox.Add(self.utpex, 0, wx.TOP, 3)

        dht_box = wx.BoxSizer(wx.HORIZONTAL)
        self.dht = wx.CheckBox(p, -1, self.localize('dht'))
        if self.abcparams['dht'] == "1":
            self.dht.SetValue(True)
        self.dhtport = wx.TextCtrl(p, -1, self.abcparams['dhtport'], wx.Point(-1, -1), wx.Size(45, -1))
        self.dhtport.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        dht_box.Add(self.dht, 0, wx.ALIGN_CENTER_VERTICAL)
        dht_box.Add(self.dhtport, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 2)
        dht_box.Add(wx.StaticText(p, -1, self.localize('restartabc')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        networkbottombox.Add(dht_box, 0, wx.TOP, 3)

        if self.utility.hasipv6:
            self.ipv6 = wx.CheckBox(p, -1, self.localize('ipv6'))
            if self.abcparams['ipv6'] == "1":
                self.ipv6.SetValue(True)
            networkbottombox.Add(self.ipv6, 0, wx.TOP, 3)

        self.upnp = wx.ComboBox(p, -1, '', (-1, -1), (-1, -1),
                                [self.localize('upnpdis'), self.localize('upnptype1'), self.localize('upnptype2')],
                                wx.CB_DROPDOWN | wx.CB_READONLY)
        self.upnp.SetSelection(int(self.abcparams['upnp']))
        upnpbox = wx.BoxSizer(wx.HORIZONTAL)
        upnpbox.Add(wx.StaticText(p, -1, self.localize('upnptxt')), 0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        upnpbox.Add(self.upnp, 0, wx.ALIGN_CENTER_VERTICAL)       
        networkbottombox.Add(upnpbox, 0, wx.TOP, 3)

        self.kickban = wx.CheckBox(p, -1, self.localize('kickban'))
        if self.abcparams['kickban'] == "1":
            self.kickban.SetValue(True)
        networkbottombox.Add(self.kickban, 0, wx.TOP, 3)

        self.notsameip = wx.CheckBox(p, -1, self.localize('security'))
        if self.abcparams['notsameip'] == "1":
            self.notsameip.SetValue(True)
        networkbottombox.Add(self.notsameip, 0, wx.TOP, 3)

        # Do or don't get scrape data
        self.scrape = wx.CheckBox(p, -1, self.localize('scrape'))
        if self.abcparams['scrape'] == "1":
            self.scrape.SetValue(True)
        networkbottombox.Add(self.scrape, 0, wx.TOP, 3)

        homebox = wx.BoxSizer(wx.HORIZONTAL)
        homebox.Add(wx.StaticText(p, -1, self.localize('homepage')), 0, wx.ALIGN_CENTER_VERTICAL)
        self.homepage = wx.TextCtrl(p, -1, self.abcparams['homepage'], wx.Point(-1, -1), wx.Size(-1, -1))
        self.homepage.Bind(wx.EVT_CHAR, self.utility.onWriteFilteredText)
        homebox.Add(self.homepage, 1, wx.ALIGN_CENTER_VERTICAL | wx.EXPAND | wx.LEFT, 5)
        self.starthome = wx.CheckBox(p, -1, self.localize('starthome'))
        if self.abcparams['starthome'] == "1":
            self.starthome.SetValue(True)
        homebox.Add(self.starthome, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 15)
        networkbottombox.Add(homebox, 0, wx.EXPAND | wx.TOP, 3)

        sizer.Add(networkbottombox, 0, wx.EXPAND | wx.TOP, 0)

        p.SetSizer(sizer)
        p.SetAutoLayout(True)
        return p

    # Torrent list Setting
    #############################
    def makeListPanel(self, parent):
        p = wx.Panel(parent, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)

        listbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, ''), wx.VERTICAL)

        stripebox = wx.BoxSizer(wx.HORIZONTAL)
        self.stripedlist = wx.CheckBox(p, -1, self.localize('stripedlist'))
        if self.abcparams['stripedlist'] == "1":
            self.stripedlist.SetValue(True)
        stripebox.Add(self.stripedlist, 0, wx.ALIGN_CENTER_VERTICAL)
        self.colstripes = wx.Window(p, -1, size = (13, 13), style = wx.SIMPLE_BORDER)
        self.colstripes.SetBackgroundColour(self.utility.colstripes)
        self.colstripes.Bind(wx.EVT_LEFT_DOWN, self.onColStripes)
        stripebox.Add(self.colstripes, 0, wx.LEFT, 2)
        listbox.Add(stripebox, 0, wx.TOP, 3)

        self.keepeta = wx.CheckBox(p, -1, self.localize('keepeta'))
        if self.abcparams['keepeta'] == "1":
            self.keepeta.SetValue(True)
        listbox.Add(self.keepeta, 0, wx.TOP, 3)

        self.sizeinmb = wx.CheckBox(p, -1, self.localize('sizeinmb'))
        self.sizeinmb.Bind(wx.EVT_CHECKBOX, self.onUpdateSizeInMB)
        if self.abcparams['sizeinmb'] == "1":
            self.sizeinmb.SetValue(True)
        listbox.Add(self.sizeinmb, 0, wx.TOP, 3)

        displayunit = wx.BoxSizer(wx.HORIZONTAL)

        displayunit.Add(wx.StaticText(p, -1, self.localize('displayunit')), 0, wx.ALIGN_LEFT)

        self.displayunitsize = wx.CheckBox(p, -1, self.localize('displayunitsize'))
        if self.abcparams['sizeinmb'] == "1":
            if self.abcparams['displayunitsize'] == "1":
                self.displayunitsize.SetValue(True)
        else:
            self.displayunitsize.SetValue(True)
            self.displayunitsize.Disable()
        displayunit.Add(self.displayunitsize, 0, wx.ALIGN_LEFT | wx.LEFT, 15)

        self.displayunitrate = wx.CheckBox(p, -1, self.localize('displayunitrate'))
        if self.abcparams['displayunitrate'] == "1":
            self.displayunitrate.SetValue(True)
        displayunit.Add(self.displayunitrate, 0, wx.ALIGN_LEFT | wx.LEFT, 15)
        listbox.Add(displayunit, 0, wx.TOP, 3)

        scrolllist = wx.BoxSizer(wx.HORIZONTAL)

        scrolllist.Add(wx.StaticText(p, -1, self.localize('scrolllist')), 0, wx.ALIGN_LEFT)

        self.scrolllistud = wx.CheckBox(p, -1, self.localize('scrolllistud'))
        if self.abcparams['scrolllistud'] == "1":
            self.scrolllistud.SetValue(True)
        scrolllist.Add(self.scrolllistud, 0, wx.ALIGN_LEFT | wx.LEFT, 15)

        self.scrolllisttb = wx.CheckBox(p, -1, self.localize('scrolllisttb'))
        if self.abcparams['scrolllisttb'] == "1":
            self.scrolllisttb.SetValue(True)
        scrolllist.Add(self.scrolllisttb, 0, wx.ALIGN_LEFT | wx.LEFT, 15)
        listbox.Add(scrolllist, 0, wx.TOP, 3)

        self.mousemode = wx.CheckBox(p, -1, self.localize('mousemode'))
        if self.abcparams['mousemode'] == "1":
            self.mousemode.SetValue(True)
        listbox.Add(self.mousemode, 0, wx.TOP, 3)

        self.scrollbelowfinished = wx.CheckBox(p, -1, self.localize('scrollbelowfinished'))
        if self.abcparams['scrollbelowfinished'] == "1":
            self.scrollbelowfinished.SetValue(True)
        listbox.Add(self.scrollbelowfinished, 0, wx.TOP, 3)

        sizer.Add(listbox, 0, wx.EXPAND | wx.TOP, 3)

        messagebox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('listmessages')), wx.VERTICAL)

        rubmsgbox = wx.BoxSizer(wx.HORIZONTAL)

        self.rubmessages = wx.CheckBox(p, -1, self.localize('rubmessages'))

        if self.abcparams['rubmessages'] == "1":
            self.rubmessages.SetValue(True)
        rubmsgbox.Add(self.rubmessages, 0, wx.ALIGN_CENTER_VERTICAL)

        self.rubmsgbutton = wx.ComboBox(p, -1, '', (-1, -1), (-1, -1),
                                        [self.localize('leftbut'), self.localize('rightbut')],
                                        wx.CB_DROPDOWN | wx.CB_READONLY)
        self.rubmsgbutton.SetSelection(int(self.abcparams['rubmsgbutton']))
        rubmsgbox.Add(self.rubmsgbutton, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 2)       
        messagebox.Add(rubmsgbox, 0, wx.TOP, 0)

        self.keeptermevtmsg = wx.CheckBox(p, -1, self.localize('keeptermevtmsg'))
        if self.abcparams['keeptermevtmsg'] == "1":
            self.keeptermevtmsg.SetValue(True)
        messagebox.Add(self.keeptermevtmsg, 0, wx.TOP, 3)

        self.erasetrackal = wx.CheckBox(p, -1, self.localize('erasetrackal'))
        if self.abcparams['erasetrackal'] == "1":
            self.erasetrackal.SetValue(True)

        messagebox.Add(self.erasetrackal, 0, wx.TOP, 3)

        displayscrape = wx.BoxSizer(wx.HORIZONTAL)

        displayscrape.Add(wx.StaticText(p, -1, self.localize('displayscrape')), 0, wx.ALIGN_LEFT)

        self.displayscrapingmsg = wx.CheckBox(p, -1, self.localize('displayscrapingmsg'))
        if self.abcparams['displayscrapingmsg'] == "1":
            self.displayscrapingmsg.SetValue(True)
        displayscrape.Add(self.displayscrapingmsg, 0, wx.ALIGN_LEFT | wx.LEFT, 15)

        self.displayscrapestatus = wx.CheckBox(p, -1, self.localize('displayscrapestatus'))
        if self.abcparams['displayscrapestatus'] == "1":
            self.displayscrapestatus.SetValue(True)
        displayscrape.Add(self.displayscrapestatus, 0, wx.ALIGN_LEFT | wx.LEFT, 15)
        messagebox.Add(displayscrape, 0, wx.TOP, 3)

        self.displaycheckstatus = wx.CheckBox(p, -1, self.localize('displaycheckstatus'))
        if self.abcparams['displaycheckstatus'] == "1":
            self.displaycheckstatus.SetValue(True)
        messagebox.Add(self.displaycheckstatus, 0, wx.TOP, 3)

        self.displaytimeout = wx.CheckBox(p, -1, self.localize('displaytimeout'))
        if self.abcparams['displaytimeout'] == "1":
            self.displaytimeout.SetValue(True)
        messagebox.Add(self.displaytimeout, 0, wx.TOP, 3)

        sizer.Add(messagebox, 0, wx.EXPAND | wx.TOP, 3)

        colourbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('torrentcolours')), wx.VERTICAL)
        coloursizer = wx.FlexGridSizer(rows = 3, cols = 4, hgap = 5, vgap = 3)
        coloursizer.Add(wx.StaticText(p, -1, self.localize('colok')), 0, wx.ALIGN_CENTER_VERTICAL)
        self.colok = wx.Window(p, -1, size = (13, 13), style = wx.SIMPLE_BORDER)
        self.colok.SetBackgroundColour(self.utility.colok)
        self.colok.Bind(wx.EVT_LEFT_DOWN, self.onColOK)
        coloursizer.Add(self.colok)
        coloursizer.Add(wx.StaticText(p, -1, self.localize('colnoinc')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 30)
        self.colnoinc = wx.Window(p, -1, size = (13, 13), style = wx.SIMPLE_BORDER)
        self.colnoinc.SetBackgroundColour(self.utility.colnoinc)
        self.colnoinc.Bind(wx.EVT_LEFT_DOWN, self.onColNoInc)
        coloursizer.Add(self.colnoinc)
        coloursizer.Add(wx.StaticText(p, -1, self.localize('colnocom')), 0, wx.ALIGN_CENTER_VERTICAL)
        self.colnocom = wx.Window(p, -1, size = (13, 13), style = wx.SIMPLE_BORDER)
        self.colnocom.SetBackgroundColour(self.utility.colnocom)
        self.colnocom.Bind(wx.EVT_LEFT_DOWN, self.onColNoCom)
        coloursizer.Add(self.colnocom)
        coloursizer.Add(wx.StaticText(p, -1, self.localize('colnocon')), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 30)
        self.colnocon = wx.Window(p, -1, size = (13, 13), style = wx.SIMPLE_BORDER)
        self.colnocon.SetBackgroundColour(self.utility.colnocon)
        self.colnocon.Bind(wx.EVT_LEFT_DOWN, self.onColNoCon)
        coloursizer.Add(self.colnocon)
        coloursizer.Add(wx.StaticText(p, -1, self.localize('coldisc')), 0, wx.ALIGN_CENTER_VERTICAL)
        self.coldisc = wx.Window(p, -1, size = (13, 13), style = wx.SIMPLE_BORDER)
        self.coldisc.SetBackgroundColour(self.utility.coldisc)
        self.coldisc.Bind(wx.EVT_LEFT_DOWN, self.onColDisc)
        coloursizer.Add(self.coldisc)
        colourbox.Add(coloursizer)
        sizer.Add(colourbox, 0, wx.EXPAND | wx.TOP, 3)

        listfontbtn = wx.Button(p, -1, self.localize('listfont'), wx.Point(-1, -1), wx.Size(-1, 20))
        self.Bind(wx.EVT_BUTTON, self.onChooseListFont, listfontbtn)
        sizer.Add(listfontbtn, 0, wx.ALIGN_CENTER_HORIZONTAL | wx.TOP | wx.BOTTOM, 6)

        p.SetSizer(sizer)
        p.SetAutoLayout(True)
        return p

    # Interface Setting
    #############################
    def makeInterfacePanel(self, parent):
        p = wx.Panel(parent, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)

        interfacebox = wx.StaticBoxSizer(wx.StaticBox(p, -1, ''), wx.VERTICAL)

        self.flattopbtn = wx.CheckBox(p, -1, self.localize('flattopbtn'))
        if self.abcparams['flattopbtn'] == "1":
            self.flattopbtn.SetValue(True)
        interfacebox.Add(self.flattopbtn, 0, wx.TOP, 3)

        self.mintray = wx.CheckBox(p, -1, self.localize('mintotray'))
        if self.abcparams['mintray'] == "1":
            self.mintray.SetValue(True)
        interfacebox.Add(self.mintray, 0, wx.TOP, 3)

        self.quickmetainfo = wx.ComboBox(p, -1, '', (-1, -1), (-1, -1),
                                         [self.localize('midbutnone'), self.localize('midbutsingle'), self.localize('midbutdouble')],
                                         wx.CB_DROPDOWN | wx.CB_READONLY)
        self.quickmetainfo.SetSelection(int(self.abcparams['quickmetainfo']))

        quickmetainfobox = wx.BoxSizer(wx.HORIZONTAL)
        quickmetainfobox.Add(wx.StaticText(p, -1, self.localize('quickmetainfo') + '\n' + self.localize('restartabc')),
                             0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        quickmetainfobox.Add(self.quickmetainfo, 0, wx.ALIGN_CENTER_VERTICAL)
        interfacebox.Add(quickmetainfobox, 0, wx.TOP, 3)

        self.centerquickdetails = wx.ComboBox(p, -1, '', (-1, -1), (-1, -1),
                                              [self.localize('quickdetrightmouse'), self.localize('quickdetcentremouse'), self.localize('quickdetcentremain')],
                                              wx.CB_DROPDOWN | wx.CB_READONLY)
        self.centerquickdetails.SetSelection(int(self.abcparams['centerquickdetails']))

        centerquickdetailsbox = wx.BoxSizer(wx.HORIZONTAL)
        centerquickdetailsbox.Add(wx.StaticText(p, -1, self.localize('centerquickdetails')),
                                    0, wx.ALIGN_CENTER_VERTICAL)
        centerquickdetailsbox.Add(self.centerquickdetails, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        interfacebox.Add(centerquickdetailsbox, 0, wx.TOP, 3)

        self.centerdetoneachother = wx.CheckBox(p, -1, self.localize('centerdetoneachother'),
                                                style = wx.CHK_3STATE | wx.CHK_ALLOW_3RD_STATE_FOR_USER)
        self.centerdetoneachother.SetToolTipString(self.localize('centerdetoneachothertt'))
        self.centerdetoneachother.Set3StateValue(int(self.abcparams['centerdetoneachother']))
        interfacebox.Add(self.centerdetoneachother, 0, wx.TOP, 3)

        self.msgindetails = wx.CheckBox(p, -1, self.localize('msgindetails'))
        if self.abcparams['msgindetails'] == "1":
            self.msgindetails.SetValue(True)
        interfacebox.Add(self.msgindetails, 0, wx.TOP, 3)

        self.countonhold = wx.CheckBox(p, -1, self.localize('countonhold'))
        if self.abcparams['countonhold'] == "1":
            self.countonhold.SetValue(True)
        interfacebox.Add(self.countonhold, 0, wx.TOP, 3)

        self.showschedcounters = wx.CheckBox(p, -1, self.localize('showschedcounters'))
        if self.abcparams['showschedcounters'] == "1":
            self.showschedcounters.SetValue(True)
        interfacebox.Add(self.showschedcounters, 0, wx.TOP, 3)

        self.countonlyalive = wx.CheckBox(p, -1, self.localize('countonlyalive'))
        if self.abcparams['countonlyalive'] == "1":
            self.countonlyalive.SetValue(True)
        interfacebox.Add(self.countonlyalive, 0, wx.TOP, 3)

        self.showactivitymax = wx.CheckBox(p, -1, self.localize('showactivitymax'))
        if self.abcparams['showactivitymax'] == "1":
            self.showactivitymax.SetValue(True)
        interfacebox.Add(self.showactivitymax, 0, wx.TOP, 3)

        displayvol = wx.BoxSizer(wx.HORIZONTAL)

        displayvol.Add(wx.StaticText(p, -1, self.localize('displayvol')), 0, wx.ALIGN_LEFT)

        self.displaydownvol = wx.CheckBox(p, -1, self.localize('displaydownvol'))
        if self.abcparams['displaydownvol'] == "1":
            self.displaydownvol.SetValue(True)
        displayvol.Add(self.displaydownvol, 0, wx.ALIGN_LEFT | wx.LEFT, 15)

        self.displayupvol = wx.CheckBox(p, -1, self.localize('displayupvol'))
        if self.abcparams['displayupvol'] == "1":
            self.displayupvol.SetValue(True)
        displayvol.Add(self.displayupvol, 0, wx.ALIGN_LEFT | wx.LEFT, 15)
        interfacebox.Add(displayvol, 0, wx.TOP, 3)

        self.savesizesonexit = wx.CheckBox(p, -1, self.localize('savesizesonexit'))
        if self.abcparams['savesizesonexit'] == "1":
            self.savesizesonexit.SetValue(True)
        interfacebox.Add(self.savesizesonexit, 0, wx.TOP, 3)

        self.confirmonclose = wx.CheckBox(p, -1, self.localize('confirmonexit'))
        if self.abcparams['confirmonclose'] == "1":
            self.confirmonclose.SetValue(True)
        interfacebox.Add(self.confirmonclose, 0, wx.TOP, 3)

        sizer.Add(interfacebox, 0, wx.EXPAND | wx.TOP, 3)

        bottombarbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('bottombar')), wx.VERTICAL)

        bottombarsizer = wx.FlexGridSizer(rows = 3, cols = 2, hgap = 20, vgap = 3)
        self.movemidbtn = wx.CheckBox(p, -1, self.localize('movemidbtn'),
                                      style = wx.CHK_3STATE | wx.CHK_ALLOW_3RD_STATE_FOR_USER)
        self.movemidbtn.SetToolTipString(self.localize('movemidbtntt'))
        self.movemidbtn.Set3StateValue(int(self.abcparams['movemidbtn']))
        bottombarsizer.Add(self.movemidbtn)
        self.schedulerbtn = wx.CheckBox(p, -1, self.localize('schedulerbtn'))
        if self.abcparams['schedulerbtn'] == "1":
            self.schedulerbtn.SetValue(True)
        bottombarsizer.Add(self.schedulerbtn)
        self.stoptrafficbtn = wx.CheckBox(p, -1, self.localize('stoptrafficbtn'))
        if self.abcparams['stoptrafficbtn'] == "1":
            self.stoptrafficbtn.SetValue(True)
        bottombarsizer.Add(self.stoptrafficbtn)
        self.scannerbtn = wx.CheckBox(p, -1, self.localize('scannerbtn'))
        if self.abcparams['scannerbtn'] == "1":
            self.scannerbtn.SetValue(True)
        bottombarsizer.Add(self.scannerbtn)
        self.stopresallbtn = wx.CheckBox(p, -1, self.localize('stopresallbtn'))
        if self.abcparams['stopresallbtn'] == "1":
            self.stopresallbtn.SetValue(True)
        bottombarsizer.Add(self.stopresallbtn)
        self.webservbtn = wx.CheckBox(p, -1, self.localize('webservbtn'))
        if self.abcparams['webservbtn'] == "1":
            self.webservbtn.SetValue(True)
        bottombarsizer.Add(self.webservbtn)
        bottombarbox.Add(bottombarsizer, 0, wx.TOP, 0)

        self.flatbotbtn = wx.CheckBox(p, -1, self.localize('flatbotbtn'))
        if self.abcparams['flatbotbtn'] == "1":
            self.flatbotbtn.SetValue(True)
        bottombarbox.Add(self.flatbotbtn, 0, wx.ALIGN_CENTER_HORIZONTAL | wx.TOP, 3)

        sizer.Add(bottombarbox, 0, wx.EXPAND | wx.TOP, 3)

        p.SetSizer(sizer)
        p.SetAutoLayout(True)
        return p

    # Misc Setting
    #############################
    def makeMiscPanel(self, parent):
        p = wx.Panel(parent, -1)
        sizer = wx.BoxSizer(wx.VERTICAL)

        downlocbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('downlocbox')), wx.VERTICAL)

        dirbox = wx.BoxSizer(wx.HORIZONTAL)
        self.defaultfolder = wx.TextCtrl(p, -1, self.abcparams['defaultfolder'])
        self.defaultfolder.Bind(wx.EVT_CHAR, self.utility.onWriteFilteredText)
        browsebtn = wx.Button(p, -1, "...", (-1, -1), (20, 20))
        self.Bind(wx.EVT_BUTTON, self.onBrowseDownDir, browsebtn)
        dirbox.Add(wx.StaticText(p, -1, self.localize('defaultfolder')), 0, wx.ALIGN_CENTER_VERTICAL)
        dirbox.Add(self.defaultfolder, 1, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        dirbox.Add(browsebtn, 0, wx.ALIGN_CENTER_VERTICAL)
        downlocbox.Add(dirbox, 0, wx.EXPAND | wx.TOP, 0)

        self.setdefaultfolder = wx.CheckBox(p, -1, self.localize('setdefaultfolder'))
        if self.abcparams['setdefaultfolder'] == "1":
            self.setdefaultfolder.SetValue(True)
        downlocbox.Add(self.setdefaultfolder, 0, wx.TOP, 3)

        self.setextdefaultfolder = wx.CheckBox(p, -1, self.localize('setextdefaultfolder'))
        if self.abcparams['setextdefaultfolder'] == "1":
            self.setextdefaultfolder.SetValue(True)
        downlocbox.Add(self.setextdefaultfolder, 0, wx.TOP, 3)

        downdestbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('downdestbox')), wx.VERTICAL)

        self.keepmagnetname = wx.CheckBox(p, -1, self.localize('keepmagnetname'))
        if self.abcparams['keepmagnetname'] == "1":
            self.keepmagnetname.SetValue(True)
        downdestbox.Add(self.keepmagnetname, 0, wx.TOP, 3)

        dupdestbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('dupdestbox')), wx.VERTICAL)

        dupdestrb0 = wx.RadioButton(p, -1, self.localize('dupdestauto'), (-1, -1), (-1, -1), wx.RB_GROUP)
        dupdestrb1 = wx.RadioButton(p, -1, self.localize('dupdestask'), (-1, -1), (-1, -1))
        dupdestrb2 = wx.RadioButton(p, -1, self.localize('dupdestkeep'), (-1, -1), (-1, -1))
        dupdestbox.Add(dupdestrb0, 0, wx.TOP, 0)
        dupdestbox.Add(dupdestrb1, 0, wx.TOP, 3)
        dupdestbox.Add(dupdestrb2, 0, wx.TOP, 3)
        self.dupdest = [dupdestrb0, dupdestrb1, dupdestrb2]
        self.dupdest[int(self.abcparams['askduplicatedest'])].SetValue(True)

        downdestbox.Add(dupdestbox, 0, wx.TOP | wx.EXPAND, 5)

        freediskspacebox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('freediskspacebox')), wx.VERTICAL)

        diskfullthbox = wx.BoxSizer(wx.HORIZONTAL)
        self.diskfullth = wx.TextCtrl(p, -1, self.abcparams['diskfullth'], wx.Point(-1, -1), wx.Size(45, -1))        
        self.diskfullth.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        diskfullthbox.Add(wx.StaticText(p, -1, self.localize('diskfullth')), 0, wx.ALIGN_CENTER_VERTICAL)
        diskfullthbox.Add(self.diskfullth, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        diskfullthbox.Add(wx.StaticText(p, -1, self.utility.mb), 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        freediskspacebox.Add(diskfullthbox, 0, wx.TOP, 0)

        self.diskfullman = wx.CheckBox(p, -1, self.localize('diskfullman'))
        if self.abcparams['diskfullman'] == "1":
            self.diskfullman.SetValue(True)
        freediskspacebox.Add(self.diskfullman, 0, wx.TOP, 3)

        movedatabox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('movedatabox')), wx.VERTICAL)

        movedatasizer = wx.BoxSizer(wx.HORIZONTAL)

        self.movedataonstatus = wx.CheckBox(p, -1, self.localize('movedataonstatus'))
        if self.abcparams['movedataonstatus'] == "1":
            self.movedataonstatus.SetValue(True)
        movedatasizer.Add(self.movedataonstatus, 0, wx.ALIGN_CENTER_VERTICAL)

        self.movedatastatus = wx.ComboBox(p, -1, '', (-1, -1), (-1, -1),
                                          [self.localize('movedatacom'), self.localize('movedatafin')],
                                          wx.CB_DROPDOWN | wx.CB_READONLY)
        self.movedatastatus.Bind(wx.EVT_COMBOBOX, self.onMoveDataStatus)
        self.movedatastatus.SetSelection(int(self.abcparams['movedatastatus']))
        movedatasizer.Add(self.movedatastatus, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)       

        self.completedresumedelaytext = wx.StaticText(p, -1, self.localize('completedresumedelay'))
        movedatasizer.Add(self.completedresumedelaytext, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 10)
        self.completedresumedelay = wx.TextCtrl(p, -1, self.abcparams['completedresumedelay'], wx.Point(-1, -1), wx.Size(30, -1))
        self.completedresumedelay.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        movedatasizer.Add(self.completedresumedelay, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
        self.completedresumedelayunit = wx.StaticText(p, -1, self.localize('l_second'))
        movedatasizer.Add(self.completedresumedelayunit, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)

        movedatabox.Add(movedatasizer, 0, wx.TOP, 0)

        self.movedataondelete = wx.CheckBox(p, -1, self.localize('movedataondelete'))
        if self.abcparams['movedataondelete'] == "1":
            self.movedataondelete.SetValue(True)
        movedatabox.Add(self.movedataondelete, 0, wx.TOP, 3)

        miscbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, ''), wx.VERTICAL)

        self.removetorrent = wx.CheckBox(p, -1, self.localize('removetorrent'))
        if self.abcparams['removetorrent'] == "1":
            self.removetorrent.SetValue(True)
        miscbox.Add(self.removetorrent, 0, wx.TOP, 3)

        self.defrentorwithdest = wx.CheckBox(p, -1, self.localize('defrentorwithdest'))
        if self.abcparams['defrentorwithdest'] == "1":
            self.defrentorwithdest.SetValue(True)
        miscbox.Add(self.defrentorwithdest, 0, wx.TOP, 3)

        self.languagefiles, languagenames = self.getLanguages()
        if not self.languagefiles:
            # Should never get here -- this means there are no valid language files found!
            sys.stderr.write("\nNO LANGUAGE FILES FOUND ! Please add a valid language file\n")
            defaultlang = ""
        else:
            try:
                index = self.languagefiles.index(self.abcparams['language_file'])
                defaultlang = languagenames[index]
            except:
                defaultlang = ""
        self.languagechoice = wx.ComboBox(p, -1, defaultlang, (-1, -1), (-1, -1), languagenames,
                                          wx.CB_DROPDOWN | wx.CB_READONLY)

        langbox = wx.BoxSizer(wx.HORIZONTAL)
        langbox.Add(wx.StaticText(p, -1, self.localize('choose_language')), 0, wx.ALIGN_CENTER_VERTICAL)
        langbox.Add(self.languagechoice, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        langbox.Add(wx.StaticText(p, -1, self.localize('restartabc')), 0, wx.ALIGN_CENTER_VERTICAL)
        miscbox.Add(langbox, 0, wx.TOP, 3)

        sizer.Add(downlocbox, 0, wx.EXPAND | wx.TOP, 3)
        sizer.Add(downdestbox, 0, wx.EXPAND | wx.TOP, 3)
        sizer.Add(movedatabox, 0, wx.EXPAND | wx.TOP, 3)
        sizer.Add(freediskspacebox, 0, wx.EXPAND | wx.TOP, 3)
        sizer.Add(miscbox, 0, wx.EXPAND | wx.TOP, 0)

        if self.utility.tweakbandwidthdistribution == 1:
            tweakbandwidthdistributionbox = wx.StaticBoxSizer(wx.StaticBox(p, -1, self.localize('bandwidthdistributiontitle')), wx.VERTICAL)

            calcdown = wx.BoxSizer(wx.HORIZONTAL)
            self.bdcalcdownth1 = wx.TextCtrl(p, -1, str(self.utility.bdcalcdownth1), wx.Point(-1, -1), wx.Size(45, -1))
            self.bdcalcdownth1.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDotMinus)
            self.bdcalcdownth2 = wx.TextCtrl(p, -1, str(self.utility.bdcalcdownth2), wx.Point(-1, -1), wx.Size(45, -1))
            self.bdcalcdownth2.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDotMinus)
            self.bdcalcdownth3 = wx.TextCtrl(p, -1, str(self.utility.bdcalcdownth3), wx.Point(-1, -1), wx.Size(45, -1))
            self.bdcalcdownth3.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDotMinus)
            calcdown.Add(wx.StaticText(p, -1, self.localize('bandwidthdistributiondown')), 0, wx.ALIGN_CENTER_VERTICAL)
            calcdown.Add(self.bdcalcdownth1, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            calcdown.Add(self.bdcalcdownth2, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            calcdown.Add(self.bdcalcdownth3, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            tweakbandwidthdistributionbox.Add(calcdown, 0, wx.TOP, 0)
            calcup = wx.BoxSizer(wx.HORIZONTAL)
            steps = wx.BoxSizer(wx.HORIZONTAL)
            self.bdcalcupth1 = wx.TextCtrl(p, -1, str(self.utility.bdcalcupth1), wx.Point(-1, -1), wx.Size(45, -1))
            self.bdcalcupth1.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDotMinus)
            self.bdcalcupth2 = wx.TextCtrl(p, -1, str(self.utility.bdcalcupth2), wx.Point(-1, -1), wx.Size(45, -1))
            self.bdcalcupth2.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDotMinus)
            self.bdcalcupth3 = wx.TextCtrl(p, -1, str(self.utility.bdcalcupth3), wx.Point(-1, -1), wx.Size(45, -1))
            self.bdcalcupth3.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDotMinus)
            calcup.Add(wx.StaticText(p, -1, self.localize('bandwidthdistributionup')), 0, wx.ALIGN_CENTER_VERTICAL)
            calcup.Add(self.bdcalcupth1, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            calcup.Add(self.bdcalcupth2, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            calcup.Add(self.bdcalcupth3, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            tweakbandwidthdistributionbox.Add(calcup, 0, wx.TOP, 3)
            self.bdstep1 = wx.TextCtrl(p, -1, str(self.utility.bdstep1), wx.Point(-1, -1), wx.Size(45, -1))
            self.bdstep1.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDotMinus)
            self.bdstep2 = wx.TextCtrl(p, -1, str(self.utility.bdstep2 ), wx.Point(-1, -1), wx.Size(45, -1))
            self.bdstep2 .Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDotMinus)
            self.bdminstepdownrate = wx.TextCtrl(p, -1, str(self.utility.bdminstepdownrate), wx.Point(-1, -1), wx.Size(45, -1))
            self.bdminstepdownrate.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDotMinus)
            self.bdminstepuprate = wx.TextCtrl(p, -1, str(self.utility.bdminstepuprate), wx.Point(-1, -1), wx.Size(45, -1))
            self.bdminstepuprate.Bind(wx.EVT_CHAR, self.utility.onWriteDigitsDotMinus)
            steps.Add(wx.StaticText(p, -1, self.localize('bandwidthdistributionsteps')), 0, wx.ALIGN_CENTER_VERTICAL)
            steps.Add(self.bdstep1, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            steps.Add(self.bdstep2, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            steps.Add(self.bdminstepdownrate, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            steps.Add(self.bdminstepuprate, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 5)
            tweakbandwidthdistributionbox.Add(steps, 0, wx.TOP, 3)

            sizer.Add(tweakbandwidthdistributionbox, 0, wx.EXPAND | wx.TOP, 3)

        assobtn = wx.Button(p, -1, self.localize('fileasso'), wx.Point(-1, -1), wx.Size(-1, 20))
        self.Bind(wx.EVT_BUTTON, self.onTorAssociate, assobtn)
        sizer.Add(assobtn, 0, wx.ALIGN_CENTER_HORIZONTAL | wx.TOP | wx.BOTTOM, 6)

        p.SetSizer(sizer)
        p.SetAutoLayout(True)
        return p

    def onMoveDataStatus(self, event):
        event.Skip()
        if event.GetSelection():
            self.completedresumedelaytext.Hide()
            self.completedresumedelay.Hide()
            self.completedresumedelayunit.Hide()
        else:
            self.completedresumedelaytext.Show()
            self.completedresumedelay.Show()
            self.completedresumedelayunit.Show()

    def onUserPeerIdMaskToggle(self, event):
        useridmaskbtn = event.GetEventObject()
        idindex = self.useridmaskbtn.index(useridmaskbtn)
        cursor = self.useridctrl[idindex].GetInsertionPoint()
        if useridmaskbtn.GetValue():
            # Edit mask
            self.userid[idindex] = self.useridctrl[idindex].GetValue()
            self.useridctrl[idindex].SetValue(self.useridmask[idindex])
            useridlen = len(self.userid[idindex]) - 1
            # Compute new cursor position
            newcursor = 0
            i = 0
            while i < cursor:
                if self.userid[idindex][i] == '\\':
                    if i != useridlen and self.userid[idindex][i + 1] == '\\':
                        newcursor += 1
                        i += 2
                    else:
                        newcursor += 1
                        i += 3
                else:
                    newcursor += 1
                    i += 1
            if i > cursor:
                newcursor -= 1
        else:
            # Edit id
            self.useridmask[idindex] = self.useridctrl[idindex].GetValue()
            self.useridctrl[idindex].SetValue(self.userid[idindex])
            useridlen = len(self.userid[idindex]) - 1
            # Compute new cursor position
            newcursor = 0
            i = 0
            while i < cursor and newcursor <= useridlen:
                if self.userid[idindex][newcursor] == '\\':
                    if newcursor != useridlen and self.userid[idindex][newcursor + 1] == '\\':
                        i += 1
                        newcursor += 2
                    else:
                        i += 1
                        newcursor += 3
                else:
                    i += 1
                    newcursor += 1
        self.useridctrl[idindex].SetInsertionPoint(newcursor)
        self.useridctrl[idindex].SetFocus()

    def onUserAgentIdRadioButton(self, event):
        for i in xrange(4):
            if self.useragentid[i].GetValue():
                self.useragentidnb = i
                break
        self.currentclientid.SetValue(self.binToString(createPeerID(useragentidnb = self.useragentidnb, idrandoms = self._idrandoms)))

    def onWriteInUserPeerId(self, event):
        # Change colour of the text of the id/mask toggle button
        event.Skip()
        useridctrl = event.GetEventObject()
        idindex = self.useridctrl.index(useridctrl)
        if self.useridmaskbtn[idindex].GetValue():
            if not useridctrl.GetValue():
                self.useridmaskbtn[idindex].SetLabel(self.localize('useridmaskbtn'))
            elif self.useridmaskbtn[idindex].GetLabel() != self.localize('useridmaskbtn2'):
                self.useridmaskbtn[idindex].SetLabel(self.localize('useridmaskbtn2'))

    def generateUserPeerId(self, idlist, useridlist, useridmasklist):
        status, idrandoms = resetUserPeerID(idlist, useridlist, useridmasklist)
        errormsg = ''
        for i in xrange(len(status)):
            if status[i] == 1:
                errormsg += self.localize('useridnb') + str(idlist[i]) + self.localize('useridhexerror') + '\n'
            elif status[i] == 2:
                errormsg += self.localize('useridnb') + str(idlist[i]) + self.localize('useridlengtherror') + '\n'
            elif status[i] == 3:
                errormsg += self.localize('useridnb') + str(idlist[i]) + self.localize('useridmasklengtherror') + '\n'
            elif status[i] == 4:
                errormsg += self.localize('useridnb') + str(idlist[i]) + self.localize('useridmaskerror') + '\n'
            elif status[i] == 5:
                errormsg += self.localize('useridnb') + str(idlist[i]) + self.localize('useridrandomerror') + '\n'
        if errormsg:
            dlg = wx.MessageDialog(self, errormsg[:-1], self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False
        for id in idlist:
            self._idrandoms[id] = idrandoms.pop(0)
        return True

    def onGenerateUserPeerId(self, event):
        if self.useragentidnb == 0:
            self._idrandoms[0] = resetPeerID()
            self.currentclientid.SetValue(self.binToString(createPeerID(useragentidnb = 0, idrandoms = self._idrandoms)))
        else:
            if self.useridmaskbtn[self.useragentidnb].GetValue():
                self.useridmask[self.useragentidnb] = self.useridctrl[self.useragentidnb].GetValue()
            else:
                self.userid[self.useragentidnb] = self.useridctrl[self.useragentidnb].GetValue()
            if not self.userid[self.useragentidnb]:
                dlg = wx.MessageDialog(self, self.localize('selecteduseridempty'), self.localize('abcokcerror'), wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
            elif self.generateUserPeerId([self.useragentidnb], [self.userid[self.useragentidnb]], [self.useridmask[self.useragentidnb]]):
                self.currentclientid.SetValue(self.binToString(createPeerID(useragentidnb = self.useragentidnb, idrandoms = self._idrandoms)))
                self.useridlastgenerate[self.useragentidnb] = self.userid[self.useragentidnb]
                self.useridmasklastgenerate[self.useragentidnb] = self.useridmask[self.useragentidnb]

    def binToString(self, binary):
        # Build a string from a binary string in input with characters > 128 and non printable characters displayed in hexa as '\XX'
        # '\' is displayed as '\\'
        outstring = ''
        for c in binary:
            if c == '\\':
                outstring += '\\\\'
            elif 31 < ord(c) < 127:
                outstring += c
            else:
                outstring += '\\' + hexlify(c)
        return outstring

    def onUpdateSizeInMB(self, event):
        event.Skip()
        if event.IsChecked():
            self.displayunitsize.Enable()
        else:
            self.displayunitsize.SetValue(True)
            self.displayunitsize.Disable()

    def onColOK(self, event):
        coldata = wx.ColourData()
        for i in xrange(16):
            coldata.SetCustomColour(i, tuple(map(int, self.abcparams['colcust%02d' % (i + 1)].split(','))))
        coldata.SetChooseFull(True)
        coldata.SetColour(self.colok.GetBackgroundColour())
        dlg = wx.ColourDialog(self, coldata)
        dlg.SetTitle(self.localize('coloktitle'))
        result = dlg.ShowModal()
        coldata = dlg.GetColourData()
        if result == wx.ID_OK:
            newcolour = coldata.GetColour()
            if newcolour != self.colok.GetBackgroundColour():
                self.colok.SetBackgroundColour(newcolour)
                self.colok.Refresh()
                self.utility.colok.Set(newcolour.Red(), newcolour.Green(), newcolour.Blue())
                for tor in self.queue.proctab:
                    if tor.abcengine:
                        tor.abcengine.updateColour()
                        if tor.infowin:
                            tor.infowin.colour.Refresh()
                        if tor.detailwin:
                            tor.detailwin.colour.Refresh()
        for i in xrange(16):
            col = coldata.GetCustomColour(i)
            self.abcparams['colcust%02d' % (i + 1)] = '%d,%d,%d' % (col.Red(), col.Green(), col.Blue())
        dlg.Destroy()

    def onColNoInc(self, event):
        coldata = wx.ColourData()
        for i in xrange(16):
            coldata.SetCustomColour(i, tuple(map(int, self.abcparams['colcust%02d' % (i + 1)].split(','))))
        coldata.SetChooseFull(True)
        coldata.SetColour(self.colnoinc.GetBackgroundColour())
        dlg = wx.ColourDialog(self, coldata)
        dlg.SetTitle(self.localize('colnoinctitle'))
        result = dlg.ShowModal()
        coldata = dlg.GetColourData()
        if result == wx.ID_OK:
            newcolour = coldata.GetColour()
            if newcolour != self.colnoinc.GetBackgroundColour():
                self.colnoinc.SetBackgroundColour(newcolour)
                self.colnoinc.Refresh()
                self.utility.colnoinc.Set(newcolour.Red(), newcolour.Green(), newcolour.Blue())
                for tor in self.queue.proctab:
                    if tor.abcengine:
                        tor.abcengine.updateColour()
                        if tor.infowin:
                            tor.infowin.colour.Refresh()
                        if tor.detailwin:
                            tor.detailwin.colour.Refresh()
        for i in xrange(16):
            col = coldata.GetCustomColour(i)
            self.abcparams['colcust%02d' % (i + 1)] = '%d,%d,%d' % (col.Red(), col.Green(), col.Blue())
        dlg.Destroy()

    def onColNoCom(self, event):
        coldata = wx.ColourData()
        for i in xrange(16):
            coldata.SetCustomColour(i, tuple(map(int, self.abcparams['colcust%02d' % (i + 1)].split(','))))
        coldata.SetChooseFull(True)
        coldata.SetColour(self.colnocom.GetBackgroundColour())
        dlg = wx.ColourDialog(self, coldata)
        dlg.SetTitle(self.localize('colnocomtitle'))
        result = dlg.ShowModal()
        coldata = dlg.GetColourData()
        if result == wx.ID_OK:
            newcolour = coldata.GetColour()
            if newcolour != self.colnocom.GetBackgroundColour():
                self.colnocom.SetBackgroundColour(newcolour)
                self.colnocom.Refresh()
                self.utility.colnocom.Set(newcolour.Red(), newcolour.Green(), newcolour.Blue())
                for tor in self.queue.proctab:
                    if tor.abcengine:
                        tor.abcengine.updateColour()
                        if tor.infowin:
                            tor.infowin.colour.Refresh()
                        if tor.detailwin:
                            tor.detailwin.colour.Refresh()
        for i in xrange(16):
            col = coldata.GetCustomColour(i)
            self.abcparams['colcust%02d' % (i + 1)] = '%d,%d,%d' % (col.Red(), col.Green(), col.Blue())
        dlg.Destroy()

    def onColNoCon(self, event):
        coldata = wx.ColourData()
        for i in xrange(16):
            coldata.SetCustomColour(i, tuple(map(int, self.abcparams['colcust%02d' % (i + 1)].split(','))))
        coldata.SetChooseFull(True)
        coldata.SetColour(self.colnocon.GetBackgroundColour())
        dlg = wx.ColourDialog(self, coldata)
        dlg.SetTitle(self.localize('colnocontitle'))
        result = dlg.ShowModal()
        coldata = dlg.GetColourData()
        if result == wx.ID_OK:
            newcolour = coldata.GetColour()
            if newcolour != self.colnocon.GetBackgroundColour():
                self.colnocon.SetBackgroundColour(newcolour)
                self.colnocon.Refresh()
                self.utility.colnocon.Set(newcolour.Red(), newcolour.Green(), newcolour.Blue())
                for tor in self.queue.proctab:
                    if tor.abcengine:
                        tor.abcengine.updateColour()
                        if tor.infowin:
                            tor.infowin.colour.Refresh()
                        if tor.detailwin:
                            tor.detailwin.colour.Refresh()
        for i in xrange(16):
            col = coldata.GetCustomColour(i)
            self.abcparams['colcust%02d' % (i + 1)] = '%d,%d,%d' % (col.Red(), col.Green(), col.Blue())
        dlg.Destroy()

    def onColDisc(self, event):
        coldata = wx.ColourData()
        for i in xrange(16):
            coldata.SetCustomColour(i, tuple(map(int, self.abcparams['colcust%02d' % (i + 1)].split(','))))
        coldata.SetChooseFull(True)
        coldata.SetColour(self.coldisc.GetBackgroundColour())
        dlg = wx.ColourDialog(self, coldata)
        dlg.SetTitle(self.localize('coldisctitle'))
        result = dlg.ShowModal()
        coldata = dlg.GetColourData()
        if result == wx.ID_OK:
            newcolour = coldata.GetColour()
            if newcolour != self.coldisc.GetBackgroundColour():
                self.coldisc.SetBackgroundColour(newcolour)
                self.coldisc.Refresh()
                self.utility.coldisc.Set(newcolour.Red(), newcolour.Green(), newcolour.Blue())
                for tor in self.queue.proctab:
                    if tor.abcengine:
                        tor.abcengine.updateColour()
                        if tor.infowin:
                            tor.infowin.colour.Refresh()
                        if tor.detailwin:
                            tor.detailwin.colour.Refresh()
        for i in xrange(16):
            col = coldata.GetCustomColour(i)
            self.abcparams['colcust%02d' % (i + 1)] = '%d,%d,%d' % (col.Red(), col.Green(), col.Blue())
        dlg.Destroy()

    def onColStripes(self, event):
        coldata = wx.ColourData()
        for i in xrange(16):
            coldata.SetCustomColour(i, tuple(map(int, self.abcparams['colcust%02d' % (i + 1)].split(','))))
        coldata.SetChooseFull(True)
        coldata.SetColour(self.colstripes.GetBackgroundColour())
        dlg = wx.ColourDialog(self, coldata)
        dlg.SetTitle(self.localize('colstripestitle'))
        result = dlg.ShowModal()
        coldata = dlg.GetColourData()
        if result == wx.ID_OK:
            newcolour = coldata.GetColour()
            if newcolour != self.colstripes.GetBackgroundColour():
                self.colstripes.SetBackgroundColour(newcolour)
                self.colstripes.Refresh()
                self.utility.colstripes.Set(newcolour.Red(), newcolour.Green(), newcolour.Blue())
                if self.abcparams['stripedlist'] == '1':
                    self.queue.updateStripesColour()
        for i in xrange(16):
            col = coldata.GetCustomColour(i)
            self.abcparams['colcust%02d' % (i + 1)] = '%d,%d,%d' % (col.Red(), col.Green(), col.Blue())
        dlg.Destroy()

    def onSetDownVolCheckNow(self, event):
        checktime = int(time())
        self.abcparams['downvolchecktime'] = str(checktime)
        self.queue.downvolchecktime = checktime
        self.queue.downvolcheckcycle = 0
        self.queue.downvol = 0
        self.queue.canswitchautodown = True
        if self.abcparams['downvolcheck'] == '1' and self.abcparams['mode'] == '2' and (self.abcparams['upvolcheck'] == '0' or self.queue.canswitchautoup):
            # Switch to auto mode
            self.parent.window.onModeToggle(mode = 1)
            self.queue.canswitchautodown = False
            self.queue.canswitchautoup = False

    def onSetDownVolCheckDate(self, event):
        setvolcheckdatedlg = SetVolCheckDateDialog(self, -1, wx.DateTimeFromTimeT(int(self.abcparams['downvolchecktime'])),
                                                   self.downvoldatebtn.ClientToScreenXY(0, 0))
        retcode = setvolcheckdatedlg.ShowModal()
        retval = setvolcheckdatedlg.datetime
        setvolcheckdatedlg.Destroy()
        if retcode == wx.ID_OK:
            now = int(time())
            checktime = retval.GetTicks()
            self.abcparams['downvolchecktime'] = str(checktime)
            self.queue.downvolchecktime = checktime
            self.queue.downvol = 0
            if now == checktime:
                self.downvolcheckcycle =  0
                self.queue.canswitchautodown = True
                if self.abcparams['downvolcheck'] == '1' and self.abcparams['mode'] == '2' and (self.abcparams['upvolcheck'] == '0' or self.queue.canswitchautoup):
                    # Switch to auto mode
                    self.parent.window.onModeToggle(mode = 1)
                    self.queue.canswitchautodown = False
                    self.queue.canswitchautoup = False
            else:
                self.downvolcheckcycle = max(-1, (now - checktime) / self.queue.downvolcountdur)

    def onSetUpVolCheckNow(self, event):
        checktime = int(time())
        self.abcparams['upvolchecktime'] = str(checktime)
        self.queue.upvolchecktime = checktime
        self.queue.upvolcheckcycle = 0
        self.queue.upvol = 0
        self.queue.canswitchautoup = True
        if self.abcparams['upvolcheck'] == '1' and self.abcparams['mode'] == '2' and (self.abcparams['downvolcheck'] == '0' or self.queue.canswitchautodown):
            # Switch to auto mode
            self.parent.window.onModeToggle(mode = 1)
            self.queue.canswitchautodown = False
            self.queue.canswitchautoup = False

    def onSetUpVolCheckDate(self, event):
        setvolcheckdatedlg = SetVolCheckDateDialog(self, -1, wx.DateTimeFromTimeT(int(self.abcparams['upvolchecktime'])),
                                                   self.upvoldatebtn.ClientToScreenXY(0, 0))
        retcode = setvolcheckdatedlg.ShowModal()
        retval = setvolcheckdatedlg.datetime
        setvolcheckdatedlg.Destroy()
        if retcode == wx.ID_OK:
            now = int(time())
            checktime = retval.GetTicks()
            self.abcparams['upvolchecktime'] = str(checktime)
            self.queue.upvolchecktime = checktime
            self.queue.upvol = 0
            if now == checktime:
                self.upvolcheckcycle =  0
                self.queue.canswitchautoup = True
                if self.abcparams['upvolcheck'] == '1' and self.abcparams['mode'] == '2' and (self.abcparams['downvolcheck'] == '0' or self.queue.canswitchautodown):
                    # Switch to auto mode
                    self.parent.window.onModeToggle(mode = 1)
                    self.queue.canswitchautodown = False
                    self.queue.canswitchautoup = False
            else:
                self.upvolcheckcycle = max(-1, (now - checktime) / self.queue.upvolcountdur)

    def onMouseWheel(self, event):
        nextpage = self.notebook.GetSelection()
        maxpage = self.notebook.GetPageCount() - 1
        if event.m_wheelRotation > 0:
            if nextpage < maxpage:
                nextpage += 1
            else:
                nextpage = 0
            self.notebook.SetSelection(nextpage)
        else:
            if nextpage > 0:
                nextpage -= 1
            else:
                nextpage = maxpage
            self.notebook.SetSelection(nextpage)

    def onCancel(self, event = None):
        # Reset list font
        if self.listfontface != self.abcparams['listfont'] or self.listfontsize != int(self.abcparams['listfontsize']) \
           or self.listfontstyle != int(self.abcparams['listfontstyle']) or self.listfontweight != int(self.abcparams['listfontweight']):
            origfont = wx.Font(int(self.abcparams['listfontsize']), wx.DEFAULT, int(self.abcparams['listfontstyle']),
                               int(self.abcparams['listfontweight']), faceName = self.abcparams['listfont'])
            self.list.EnsureVisible(0)
            self.list.SetFont(origfont)
            self.list.Refresh()
            if self.parent.abcfind is not None:
                self.parent.abcfind.window.list.EnsureVisible(0)
                self.parent.abcfind.window.list.SetFont(origfont)
                self.parent.abcfind.window.list.Refresh()

        # Reset torrents and stripes colours
        colhaschanged = strcolhaschanged = False
        rgb = map(int, self.abcparams['colok'].split(','))
        if self.colok.GetBackgroundColour() != wx.Colour(rgb[0], rgb[1], rgb[2]):
            colhaschanged = True
            self.utility.colok.Set(rgb[0], rgb[1], rgb[2])
        rgb = map(int, self.abcparams['colnoinc'].split(','))
        if self.colnoinc.GetBackgroundColour() != wx.Colour(rgb[0], rgb[1], rgb[2]):
            colhaschanged = True
            self.utility.colnoinc.Set(rgb[0], rgb[1], rgb[2])
        rgb = map(int, self.abcparams['colnocom'].split(','))
        if self.colnocom.GetBackgroundColour() != wx.Colour(rgb[0], rgb[1], rgb[2]):
            colhaschanged = True
            self.utility.colnocom.Set(rgb[0], rgb[1], rgb[2])
        rgb = map(int, self.abcparams['colnocon'].split(','))
        if self.colnocon.GetBackgroundColour() != wx.Colour(rgb[0], rgb[1], rgb[2]):
            colhaschanged = True
            self.utility.colnocon.Set(rgb[0], rgb[1], rgb[2])
        rgb = map(int, self.abcparams['coldisc'].split(','))
        if self.coldisc.GetBackgroundColour() != wx.Colour(rgb[0], rgb[1], rgb[2]):
            colhaschanged = True
            self.utility.coldisc.Set(rgb[0], rgb[1], rgb[2])
        rgb = map(int, self.abcparams['colstripes'].split(','))
        if self.colstripes.GetBackgroundColour() != wx.Colour(rgb[0], rgb[1], rgb[2]):
            strcolhaschanged = True
            self.utility.colstripes.Set(rgb[0], rgb[1], rgb[2])
        if strcolhaschanged and self.abcparams['stripedlist'] == '1':
            if colhaschanged:
                for i in xrange(self.list.GetItemCount()):
                    self.queue.updateItemDisplay(i)
                    tor = self.queue.proctab[i]
                    if tor.abcengine:
                        if tor.infowin:
                            tor.infowin.colour.Refresh()
                        if tor.detailwin:
                            tor.detailwin.colour.Refresh()
            else:
                for i in xrange(1, self.list.GetItemCount(), 2):
                    self.queue.updateItemDisplay(i)
        elif colhaschanged:
            for tor in self.queue.proctab:
                if tor.abcengine:
                    tor.abcengine.updateColour()
                    if tor.infowin:
                        tor.infowin.colour.Refresh()
                    if tor.detailwin:
                        tor.detailwin.colour.Refresh()

        if self.advancedMenuBox is not None:
            self.advancedMenuBox.Close()

        self.EndModal(wx.ID_CANCEL)

    def onAdvSetting(self, event):
        if self.advancedMenuBox is not None:
            self.advancedMenuBox.Raise()
            return
        self.advancedMenuBox = ABCAdvancedOptionWindow(self)

    def onBrowseDownDir(self, event):
        folderdialog = wx.DirDialog(self, self.localize('choosedefaultdownloadfolder'), 
                                    style = wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        if folderdialog.ShowModal() == wx.ID_OK:
            self.defaultfolder.SetValue(folderdialog.GetPath())
        folderdialog.Destroy()

    def onTorAssociate(self, event):
        if self.utility.mainport == 56666:
            commandlinemainport = ''
        else:
            commandlinemainport = ' -p ' + str(self.utility.mainport)
        # Associate .torrent files with ABC_OKC
        key = RegCreateKey(win32con.HKEY_CLASSES_ROOT, '.torrent')
        RegSetValueEx(key, '', 0, win32con.REG_SZ, 'bittorrent')
        RegSetValueEx(key, 'Content Type', 0, win32con.REG_SZ, 'application/x-bittorrent')
        RegCloseKey(key)
        key = RegCreateKey(win32con.HKEY_CLASSES_ROOT, 'MIME\\Database\\Content Type\\application/x-bittorrent')
        RegSetValueEx(key, 'Extension', 0, win32con.REG_SZ, '.torrent')
        RegCloseKey(key)
        key = RegCreateKey(win32con.HKEY_CLASSES_ROOT, 'bittorrent')
        RegSetValueEx(key, '', 0, win32con.REG_SZ, 'TORRENT File')
        RegSetValueEx(key, 'DefaultIcon', 0, win32con.REG_SZ, path.join(self.utility.abcpath.encode(sysencoding), 'torrenticon.ico'))
        RegSetValueEx(key, 'EditFlags', 0, win32con.REG_BINARY, '\x00\x00\x01\x00')
        RegCloseKey(key)
        key = RegCreateKey(win32con.HKEY_CLASSES_ROOT, 'bittorrent\\shell')
        RegSetValueEx(key, '', 0, win32con.REG_SZ, 'open')
        RegCloseKey(key)
        key = RegCreateKey(win32con.HKEY_CLASSES_ROOT, 'bittorrent\\shell\\open\\command')
        RegSetValueEx(key, '', 0, win32con.REG_SZ, '"' + path.join(self.utility.abcpath.encode(sysencoding), 'abc.exe') + '"' + commandlinemainport + ' -a "%1"')
        RegCloseKey(key)

        # Associate magnet protocol with ABC_OKC
        key = RegCreateKey(win32con.HKEY_CLASSES_ROOT, 'magnet')
        RegSetValueEx(key, '', 0, win32con.REG_SZ, 'URL:Magnet Protocol')
        RegSetValueEx(key, 'URL Protocol', 0, win32con.REG_SZ, '')
        RegCloseKey(key)
        key = RegCreateKey(win32con.HKEY_CLASSES_ROOT, 'magnet\\DefaultIcon')
        RegSetValueEx(key, '', 0, win32con.REG_SZ, path.join(self.utility.abcpath.encode(sysencoding), 'torrenticon.ico'))
        RegCloseKey(key)
        key = RegCreateKey(win32con.HKEY_CLASSES_ROOT, 'magnet\\shell\\open\\command')
        RegSetValueEx(key, '', 0, win32con.REG_SZ, '"' + path.join(self.utility.abcpath.encode(sysencoding), 'abc.exe') + '"' + commandlinemainport + ' -a "%1"')
        RegCloseKey(key)

        dlg = wx.MessageDialog(self, self.localize('fileassook'), self.localize('success'), wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def onChooseListFont(self, event):
        # Choose font for lists
        data = wx.FontData()
        data.EnableEffects(False)
        data.SetInitialFont(wx.Font(self.listfontsize, wx.DEFAULT, self.listfontstyle, self.listfontweight, faceName = self.listfontface))

        dlg = wx.FontDialog(self, data)
        if dlg.ShowModal() == wx.ID_OK:
            data = dlg.GetFontData()
            font = data.GetChosenFont()
            newlistfontface = font.GetFaceName()
            newlistfontsize = font.GetPointSize()
            newlistfontstyle = font.GetStyle()
            newlistfontweight = font.GetWeight()

            if newlistfontface != self.listfontface or newlistfontsize != self.listfontsize \
               or newlistfontstyle != self.listfontstyle or newlistfontweight != self.listfontweight:
                self.listfontface = newlistfontface
                self.listfontsize = newlistfontsize
                self.listfontstyle = newlistfontstyle
                self.listfontweight = newlistfontweight
                self.list.EnsureVisible(0)
                self.list.SetFont(font)
                self.list.Refresh()
                if self.parent.abcfind is not None:
                    self.parent.abcfind.window.list.EnsureVisible(0)
                    self.parent.abcfind.window.list.SetFont(font)
                    self.parent.abcfind.window.list.Refresh()

        dlg.Destroy()

    def onApply(self, event):
        # Set new value to abcparams
        ##############################
        errortype = False
        try:
            down_vol_max = float(self.downvolmax.GetValue())
        except:
            errortype = True
        if errortype or down_vol_max < 0:
            dlg = wx.MessageDialog(self, self.localize('downvolmaxwarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        errortype = False
        try:
            down_vol_count_dur = int(self.downvolcountdur.GetValue())
        except:
            errortype = True
        if errortype or down_vol_count_dur < 1:
            dlg = wx.MessageDialog(self, self.localize('downvolcountdurwarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        errortype = False
        try:
            numsim = int(self.numsimtext.getValue())
        except:
            errortype = True
        if errortype or numsim < 0 or numsim > 99:
            dlg = wx.MessageDialog(self, self.localize('numsimintwarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        errortype = False
        try:
            upload_rate = int(self.uploadrate.GetValue())
            seedupload_rate = int(self.seeduploadrate.GetValue())
        except:
            errortype = True
        if errortype or upload_rate < self.utility.minuprate and upload_rate != 0 or seedupload_rate < self.utility.minuprate and seedupload_rate != 0:
            dlg = wx.MessageDialog(self, self.localize('uploadrateminwarning') % self.utility.minuprate,
                                   self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        errortype = False
        try:
            up_vol_max = float(self.upvolmax.GetValue())
        except:
            errortype = True
        if errortype or up_vol_max < 0:
            dlg = wx.MessageDialog(self, self.localize('upvolmaxwarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        errortype = False
        try:
            up_vol_count_dur = int(self.upvolcountdur.GetValue())
        except:
            errortype = True
        if errortype or up_vol_count_dur < 1:
            dlg = wx.MessageDialog(self, self.localize('upvolcountdurwarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        errortype = False
        try:
            minport = int(self.minport.GetValue())
            maxport = int(self.maxport.GetValue())
        except:
            errortype = True
        if errortype or minport < 0 or maxport < 0 or minport > 65535 or maxport > 65535 or minport > maxport:
            dlg = wx.MessageDialog(self, self.localize('portrangewarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        # Compute and check user client ids if id or mask has changed
        idlist = []
        useridlist = []
        useridmasklist = []
        for i in xrange(1, 4):
            if self.useridmaskbtn[i].GetValue():
                self.useridmask[i] = self.useridctrl[i].GetValue()
            else:
                self.userid[i] = self.useridctrl[i].GetValue()
            if self.userid[i] != self.useridlastgenerate[i] \
               or self.useridmask[i] != self.useridmasklastgenerate[i]:
                idlist.append(i)
                useridlist.append(self.userid[i])
                useridmasklist.append(self.useridmask[i])
        if not self.generateUserPeerId(idlist, useridlist, useridmasklist):
            return False

        # Selected user id must not be empty
        if self.useragentidnb != 0 and not self.userid[self.useragentidnb]:
            dlg = wx.MessageDialog(self, self.localize('selecteduseridempty'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        errortype = False
        try:
            dhtport = int(self.dhtport.GetValue())
        except:
            errortype = True
        if errortype or dhtport < 0  or dhtport > 65535:
            dlg = wx.MessageDialog(self, self.localize('dhtportrangewarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        errortype = False
        try:
            urmthstart = float(self.urmthresholdstarttext.GetValue())
            urmthstop = float(self.urmthresholdstoptext.GetValue())
        except:
            errortype = True
        # Check if urmthresholdstart >= urmthresholdstop
        if errortype or urmthstart < urmthstop:
            dlg = wx.MessageDialog(self, self.localize('urmthresholdwarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        errortype = False
        try:
            urmdel = int(self.urmdelaystarttext.GetValue())
        except:
            errortype = True
        # Check if urmdelay >= 1 s
        if errortype or urmdel < 1:
            dlg = wx.MessageDialog(self, self.localize('urmdelayrangewarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        errortype = False
        try:
            urmdelstop = int(self.urmdelaystoptext.GetValue())
        except:
            errortype = True
        # Check if urmdelaystop >= 1 s
        if errortype or urmdelstop < 1:
            dlg = wx.MessageDialog(self, self.localize('urmdelayrangewarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        errortype = False
        try:
            urmtorrentstartdel = int(self.urmtorrentstartdelaytext.GetValue())
        except:
            errortype = True
        # Check if urmtorrentstartdel >= 1 s
        if errortype or urmtorrentstartdel < 1:
            dlg = wx.MessageDialog(self, self.localize('urmdelayrangewarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        errortype = False
        try:
            urmtorrentstopdel = int(self.urmtorrentstopdelaytext.GetValue())
        except:
            errortype = True
        # Check if urmtorrentstopdel >= 1 s
        if errortype or urmtorrentstopdel < 1:
            dlg = wx.MessageDialog(self, self.localize('urmdelayrangewarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        if (self.setdefaultfolder.GetValue() or self.setextdefaultfolder.GetValue()) and self.defaultfolder.GetValue() == '':
            dlg = wx.MessageDialog(self, self.localize('nodefaultdowndirwarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        # Check if default download folder is a valid Windows name
        defdestfolder = self.defaultfolder.GetValue()
        if sys.platform == 'win32' and (self.setdefaultfolder.GetValue() and self.abcparams['setdefaultfolder'] == "0" \
           or self.setextdefaultfolder.GetValue() and self.abcparams['setextdefaultfolder'] == "0" \
           or defdestfolder != self.abcparams['defaultfolder']):
            # We erase the final '\' except for a path like 'X:\'
            if defdestfolder and defdestfolder[-1] == '\\' and (len(defdestfolder) < 2 or defdestfolder[-2] != ':'):
                defdestfolder = defdestfolder[:-1]
            defdestfolder = defdestfolder.rstrip(whitespace + '.')
            if not self.utility.checkWinPath(self, defdestfolder):
                return False

        if self.utility.tweakdynmaxuprate == 1:
            try:
                urmupfromdownA = float(self.urmupfromdownAtext.GetValue())
                urmupfromdownB = float(self.urmupfromdownBtext.GetValue())
                urmupfromdownC = float(self.urmupfromdownCtext.GetValue())
                urmupfromdownD = float(self.urmupfromdownDtext.GetValue())
                urmupfromdownE = float(self.urmupfromdownEtext.GetValue())
                urmupfromdownF = float(self.urmupfromdownFtext.GetValue())
                urmupfromdownG = float(self.urmupfromdownGtext.GetValue())
            except: # not float or integer
                dlg = wx.MessageDialog(self, self.localize('urmfactrangewarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
                return False

        if self.utility.tweakbandwidthdistribution == 1:
            try:
                bdcalcdownth1 = float(self.bdcalcdownth1.GetValue())
                bdcalcdownth2 = float(self.bdcalcdownth2.GetValue())
                bdcalcdownth3 = float(self.bdcalcdownth3.GetValue())
                bdcalcupth1 = float(self.bdcalcupth1.GetValue())
                bdcalcupth2 = float(self.bdcalcupth2.GetValue())
                bdcalcupth3 = float(self.bdcalcupth3.GetValue())
                bdstep1 = float(self.bdstep1.GetValue())
                bdstep2 = float(self.bdstep2.GetValue())
                bdminstepdownrate = float(self.bdminstepdownrate.GetValue())
                bdminstepuprate = float(self.bdminstepuprate.GetValue())
            except: # not float
                dlg = wx.MessageDialog(self, self.localize('bdparamrangewarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
                return False

        urmmaxtor = int(self.urmmaxtorrenttext.getValue())
        activitymaxtor = int(self.activitymaxtext.getValue())
        # Check if "max number of simultaneous downloads" + "URM number of torrent" (if URM is running) is lower
        # than the available number of ports. 
        totalmaxdownload = numsim
        if self.activityextra.GetValue():
            totalmaxdownload += activitymaxtor
        if self.urm.GetValue():
            totalmaxdownload += urmmaxtor
        self.queue.reservedports = maxport - minport + 1
        if totalmaxdownload > self.queue.reservedports:
            dlg = wx.MessageDialog(self, self.localize('maxsimdownloadwarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        try:
            actthdown = float(self.activitythdown.GetValue())
            actthup = float(self.activitythup.GetValue())
            actdelay = int(self.activitydelay.GetValue())
        except:
            dlg = wx.MessageDialog(self, self.localize('activityrangewarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False
        if actthdown < 0 or actthup < 0 or actdelay < 0:
            dlg = wx.MessageDialog(self, self.localize('activityrangewarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        try:
            diskfullthint = int(self.diskfullth.GetValue())
        except:
            dlg = wx.MessageDialog(self, self.localize('diskfullthintwarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        if diskfullthint < 5:
            dlg = wx.MessageDialog(self, self.localize('diskfullthminwarning'), self.localize('abcokcerror'), wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
            return False

        ############################################### End for checkings ###################################################################"


        self.abcparams['maxdownloadrate'] = str(int(self.downloadrate.GetValue()))

        if self.downvolcheck.GetValue():
            self.abcparams['downvolcheck'] = "1"
        else:
            self.abcparams['downvolcheck'] = "0"

        self.abcparams['downvolmax'] = str(down_vol_max)
        self.queue.downvolmax = int(down_vol_max * 1073741824)

        self.abcparams['downvolcountdur'] = str(down_vol_count_dur)
        self.queue.downvolcountdur = down_vol_count_dur * 3600

        if not self.urmdynmaxuprate.GetValue() and upload_rate > int(self.abcparams['maxuploadrate']) \
           and self.queue.torrentdownloading:
            self.queue.urminitialstartingtime = clock()
        self.abcparams['maxuploadrate'] = str(upload_rate)

        if self.upvolcheck.GetValue():
            self.abcparams['upvolcheck'] = "1"
        else:
            self.abcparams['upvolcheck'] = "0"

        self.abcparams['upvolmax'] = str(up_vol_max)
        self.queue.upvolmax = int(up_vol_max * 1073741824)

        self.abcparams['upvolcountdur'] = str(up_vol_count_dur)
        self.queue.upvolcountdur = up_vol_count_dur * 3600

        # Switch to auto mode if mode == 2 and the new max volume limits are not reached
        if self.abcparams['mode'] == "2" and \
            (self.abcparams['downvolcheck'] == "0" or self.queue.downvol < self.queue.downvolmax) \
            and (self.abcparams['upvolcheck'] == "0" or self.queue.upvol < self.queue.upvolmax):
            self.parent.window.onModeToggle(mode = 1)

        if not self.urmdynmaxuprate.GetValue() and seedupload_rate > int(self.abcparams['maxseeduploadrate']) \
           and not self.queue.torrentdownloading:
            self.queue.urminitialstartingtime = clock()
        self.abcparams['maxseeduploadrate'] = str(seedupload_rate)

        if self.urmdynmaxuprate.GetValue():
            if self.abcparams['urmdynmaxuprate'] == "0" or int(self.urmmaxsysupratetext.GetValue()) > int(self.abcparams['urmmaxsysuprate']):
                self.queue.urminitialstartingtime = clock()
            if self.abcparams['urmdynmaxuprate'] == "0":
                # Store the current static max up rate into the list that is used to calculate the mean dyn upload rate
                # This will provide a smooth transition in time from the static value to the dynamic value
                # That's better than leaving old erroneous values in the mean list, or to store null values into it.
                if self.queue.torrentdownloading:
                    # Static overall maximum up rate when downloading
                    staticuprate = float(self.abcparams['maxuploadrate'])        
                else:
                    # Static overall maximum up rate when seeding
                    staticuprate = float(self.abcparams['maxseeduploadrate'])
                for i in xrange(10):
                    self.queue.pastdynmaxuprate[i] = staticuprate
                self.queue.dynmaxuprate = staticuprate            
                self.abcparams['urmdynmaxuprate'] = "1"
        elif self.abcparams['urmdynmaxuprate'] == "1":
            self.abcparams['urmdynmaxuprate'] = "0"
            self.queue.urminitialstartingtime = clock()

        self.abcparams['urmmaxsysuprate'] = str(int(self.urmmaxsysupratetext.GetValue()))

        if self.utility.tweakdynmaxuprate == 1:
            self.utility.urmupfromdownA = urmupfromdownA
            self.utility.urmupfromdownB = urmupfromdownB
            self.utility.urmupfromdownC = urmupfromdownC
            self.utility.urmupfromdownD = urmupfromdownD
            self.utility.urmupfromdownE = urmupfromdownE
            self.utility.urmupfromdownF = urmupfromdownF
            self.utility.urmupfromdownG = urmupfromdownG
            self.abcparams['urmupfromdownA'] = str(urmupfromdownA)
            self.abcparams['urmupfromdownB'] = str(urmupfromdownB)
            self.abcparams['urmupfromdownC'] = str(urmupfromdownC)
            self.abcparams['urmupfromdownD'] = str(urmupfromdownD)
            self.abcparams['urmupfromdownE'] = str(urmupfromdownE)
            self.abcparams['urmupfromdownF'] = str(urmupfromdownF)
            self.abcparams['urmupfromdownG'] = str(urmupfromdownG)

        if self.utility.tweakbandwidthdistribution == 1:
            self.utility.bdcalcdownth1 = bdcalcdownth1
            self.utility.bdcalcdownth2 = bdcalcdownth2
            self.utility.bdcalcdownth3 = bdcalcdownth3
            self.utility.bdcalcupth1 = bdcalcupth1
            self.utility.bdcalcupth2 = bdcalcupth2
            self.utility.bdcalcupth3 = bdcalcupth3
            self.utility.bdstep1 = bdstep1
            self.utility.bdstep2 = bdstep2
            self.utility.bdminstepdownrate = bdminstepdownrate
            self.utility.bdminstepuprate = bdminstepuprate
            self.abcparams['bdcalcdownth1'] = str(bdcalcdownth1)
            self.abcparams['bdcalcdownth2'] = str(bdcalcdownth2)
            self.abcparams['bdcalcdownth3'] = str(bdcalcdownth3)
            self.abcparams['bdcalcupth1'] = str(bdcalcupth1)
            self.abcparams['bdcalcupth2'] = str(bdcalcupth2)
            self.abcparams['bdcalcupth3'] = str(bdcalcupth3)
            self.abcparams['bdstep1'] = str(bdstep1)
            self.abcparams['bdstep2'] = str(bdstep2)
            self.abcparams['bdminstepdownrate'] = str(bdminstepdownrate)
            self.abcparams['bdminstepuprate'] = str(bdminstepuprate)

        self.abcparams['prioritizelocaldown'] = str(int(self.prioritizelocaldown.GetValue()))

        self.abcparams['prioritizelocal'] = str(int(self.prioritizelocal.GetValue()))

        self.abcparams['fastresume'] = str(int(self.fastresume.GetValue()))

        self.abcparams['fastresumefirst'] = str(int(self.fastresumefirst.GetValue()))

        self.abcparams['sbqueued'] = str(int(self.sbqueued.GetValue()))

        self.abcparams['schedonhold'] = str(int(self.schedonhold.GetValue()))

        self.abcparams['schedstopsman'] = str(int(self.schedstopsman.GetValue()))

        self.abcparams['lockedcompletedcheck'] = str(int(self.lockedcompletedcheck.GetValue()))

        self.abcparams['lockedcompletedmax'] = str(int(self.lockedcompletedmaxtext.getValue()))

        self.abcparams['stoprejected'] = str(int(self.stoprejected.GetValue()))

        self.abcparams['rejectexceptions'] = self.rejectexceptions.GetValue()
        self.queue.rejectexceptions[:] = []
        for s in self.rejectexceptions.GetValue().split():
            self.queue.rejectexceptions.append(s)

        self.abcparams['timeoutwhenextra'] = str(int(self.timeoutwhenextra.GetValue()))

        self.abcparams['canceltimeouts'] = str(int(self.canceltimeouts.GetValue()))

        self.abcparams['canceltimeoutsmsg'] = self.canceltimeoutsmsg.GetValue()
        self.queue.canceltimeoutsmsg[:] = []
        for s in self.canceltimeoutsmsg.GetValue().split():
            self.queue.canceltimeoutsmsg.append(s)

        self.abcparams['sbtimeout'] = str(int(self.sbtimeout.GetValue()))

        self.abcparams['sbtimeoutvalue'] = str(int(self.sbtimeoutvalue.GetValue()))

        self.abcparams['sbtimeoutaction'] = str(self.sbtimeoutaction.GetSelection())

        self.abcparams['torrentwhencompleted'] = str(self.torrentwhencompleted.GetSelection())

        self.abcparams['torrentwhenfinished'] = str(self.torrentwhenfinished.GetSelection())

        self.abcparams['sortbytime'] = str(int(self.sortbytime.GetValue()))

        self.abcparams['sorttoronclose'] = str(int(self.sorttoronclose.GetValue()))

        self.abcparams['urm'] = str(int(self.urm.GetValue()))

        self.abcparams['urmonhold'] = str(int(self.urmonhold.GetValue()))

        self.abcparams['urmstopsman'] = str(int(self.urmstopsman.GetValue()))

        self.abcparams['urmstopsonlyactive'] = str(int(self.urmstopsonlyactive.GetValue()))

        self.abcparams['urmstopsonlylowprio'] = str(int(self.urmstopsonlylowprio.GetValue()))

        self.abcparams['urmstoponpause'] = str(int(self.urmstoponpause.GetValue()))

        self.abcparams['urmlowpriority'] = str(int(self.urmlowpriority.GetValue()))

        self.abcparams['urmthresholdstart'] = str(urmthstart)
        self.abcparams['urmthresholdstop'] = str(urmthstop)
        self.abcparams['urmmaxtorrent'] = str(urmmaxtor)
        self.abcparams['urmdelay'] = str(urmdel)
        self.abcparams['urmdelaystop'] = str(urmdelstop)
        self.abcparams['urmtorrentstartdelay'] = str(urmtorrentstartdel)
        self.abcparams['urmtorrentstopdelay'] = str(urmtorrentstopdel)

        self.abcparams['scheddonthalt'] = str(int(self.scheddonthalt.GetValue()))
        self.abcparams['scheddonthaltuprategap'] = str(int(self.scheddonthaltuprategap.GetValue()))

        self.abcparams['activitythdown'] = str(actthdown)
        self.abcparams['activitythup'] = str(actthup)
        self.abcparams['activitydelay'] = str(actdelay)

        if self.activityextra.GetValue():
            self.abcparams['activityextra'] = "1"
            self.queue.activitymax = activitymaxtor
        else:
            self.abcparams['activityextra'] = "0"
            self.queue.activitymax = 0

        self.abcparams['activitymax'] = str(activitymaxtor)

        extratobestopped = self.extratobestoppedactive.Get3StateValue()
        if self.extratobestoppedinactive.GetValue():
            extratobestopped += 10
        self.abcparams['extratobestopped'] = str(extratobestopped)
        self.queue.extratobestoppedmode = extratobestopped
        if not extratobestopped:
            self.queue.extratobestopped = 0
            self.queue.freshactivetorrents = []

        self.abcparams['dontcheckseedactivity'] = str(int(self.dontcheckseedactivity.GetValue()))

        self.abcparams['minport'] = str(minport)
        self.abcparams['maxport'] = str(maxport)
        self.abcparams['upnp'] = str(self.upnp.GetSelection())

        self.abcparams['useragentid'] = str(self.useragentidnb)
        updateIdRandoms(self._idrandoms)
        self.currentclientid.SetValue(self.binToString(createPeerID(useragentidnb = self.useragentidnb)))
        for i in xrange(1, 4):
            self.abcparams['useragent' + str(i)] = self.useragentctrl[i].GetValue()
            self.abcparams['userid' + str(i)] = self.userid[i]
            self.abcparams['useridmask' + str(i)] = self.useridmask[i]
        self.useridlastgenerate = self.userid[:]
        self.useridmasklastgenerate = self.useridmask[:]
        # Set HTTP useragent in BitTornado
        if self.useragentidnb == 0:
            useragent = self.parent.defuseragent
        else:
            useragent = self.abcparams['useragent' + str(self.useragentidnb)]
        setUserAgent(useragent)
        # Set peer ID in BitTornado
        setPeerID()

        if CRYPTO_OK:
            for i in xrange(4):
                if self.crypto[i].GetValue():
                    break
            self.abcparams['crypto'] = str(i)

        self.abcparams['numsimdownload'] = str(numsim)
        self.queue.maxnumsim = numsim

        self.abcparams['defaultpriority'] = str(self.defaultpriority.GetSelection())

        self.abcparams['diskfullth'] = str(diskfullthint)

        self.abcparams['completedresumedelay'] = str(int(self.completedresumedelay.GetValue()))

        selectedlanguage = self.languagechoice.GetSelection()
        if selectedlanguage > -1:
            self.abcparams['language_file'] = self.languagefiles[selectedlanguage]
        else:
            self.abcparams['language_file'] = ""

        self.abcparams['removetorrent'] = str(int(self.removetorrent.GetValue()))

        if self.keepeta.GetValue():
            if self.abcparams['keepeta'] == "0":
                self.abcparams['keepeta'] = "1"
                rank = self.parent.guiman.getRankfromID(8)
                if rank != -1:
                    for index in xrange(len(self.queue.proctab)):
                        torrent = self.queue.proctab[index]
                        if not torrent.abcengine or torrent.status == 'pause' or torrent.status == 'onhold':
                            if torrent.status != 'finished':
                                if torrent.complete:
                                    if torrent.uploadoption == "0":
                                        self.list.SetStringItem(index, rank, self.localize('etaS') + '     oo')
                                    elif torrent.uploadoption == "1":
                                        self.list.SetStringItem(index, rank, self.localize('etaS') + " "
                                                                + self.utility.eta_value(torrent.seedingtimeleft))
                                    else:
                                        self.list.SetStringItem(index, rank, self.localize('etaS') + "      ?")
                                else:
                                    self.list.SetStringItem(index, rank, self.localize('etaD') + "      ?")
        elif self.abcparams['keepeta'] == "1":
            self.abcparams['keepeta'] = "0"
            rank = self.parent.guiman.getRankfromID(8)
            if rank != -1:
                for index in xrange(len(self.queue.proctab)):
                    torrent = self.queue.proctab[index]
                    if not torrent.abcengine or torrent.status == 'pause' or torrent.status == 'onhold':
                        self.list.SetStringItem(index, rank, "")

        self.abcparams['defaultfolder'] = defdestfolder
        self.abcparams['setdefaultfolder'] = str(int(self.setdefaultfolder.GetValue()))
        self.abcparams['setextdefaultfolder'] = str(int(self.setextdefaultfolder.GetValue()))

        self.abcparams['movedataonstatus'] = str(int(self.movedataonstatus.GetValue()))

        self.abcparams['movedatastatus'] = str(self.movedatastatus.GetSelection())

        self.abcparams['movedataondelete'] = str(int(self.movedataondelete.GetValue()))

        self.abcparams['keepmagnetname'] = str(int(self.keepmagnetname.GetValue()))

        for i in xrange(3):
            if self.dupdest[i].GetValue():
                break
        self.abcparams['askduplicatedest'] = str(i)

        self.abcparams['defrentorwithdest'] = str(int(self.defrentorwithdest.GetValue()))

        self.abcparams['diskfullman'] = str(int(self.diskfullman.GetValue()))

        tbstyle = -1
        if self.flattopbtn.GetValue():
            if self.abcparams['flattopbtn'] == "0":
                self.abcparams['flattopbtn'] = "1"
                tbstyle = wx.TB_HORIZONTAL | wx.NO_BORDER | wx.TB_TEXT | wx.TB_FLAT
        elif self.abcparams['flattopbtn'] == "1":
            self.abcparams['flattopbtn'] = "0"
            tbstyle = wx.TB_HORIZONTAL | wx.NO_BORDER | wx.TB_TEXT
        if tbstyle != -1:
            self.parent.setToolbarStyle(tbstyle)
            self.parent.window.searchpanel.html.setToolbarStyle(tbstyle | wx.TB_NODIVIDER)
            self.parent.window.rsspanel.setToolbarStyle(tbstyle | wx.TB_NODIVIDER)

        self.abcparams['mintray'] = str(int(self.mintray.GetValue()))

        self.abcparams['quickmetainfo'] = str(self.quickmetainfo.GetSelection())

        self.abcparams['centerquickdetails'] = str(self.centerquickdetails.GetSelection())

        self.abcparams['centerdetoneachother'] = str(self.centerdetoneachother.Get3StateValue())

        if self.sizeinmb.GetValue() != (self.abcparams['sizeinmb'] == "1") \
           or self.displayunitsize.GetValue() != (self.abcparams['displayunitsize'] == "1"):
            self.abcparams['sizeinmb'] = str(int(self.sizeinmb.GetValue()))
            self.abcparams['displayunitsize'] = str(int(self.displayunitsize.GetValue()))
            ranksize = self.parent.guiman.getRankfromID(9)
            rankdownsize = self.parent.guiman.getRankfromID(18)
            rankupsize = self.parent.guiman.getRankfromID(19)
            for index in xrange(len(self.queue.proctab)):
                torrent = self.queue.proctab[index]
                if ranksize != -1:
                    self.list.SetStringItem(index, ranksize, self.utility.formatedSize(torrent.requestedsize))
                if not torrent.abcengine or torrent.status == 'pause' or torrent.status == 'onhold':
                    if rankdownsize != -1:
                        self.list.SetStringItem(index, rankdownsize, self.utility.formatedSize(torrent.downsize))
                    if rankupsize != -1:
                        self.list.SetStringItem(index, rankupsize, self.utility.formatedSize(torrent.upsize))
                    # Update display format for already opened advanced details window
                    if torrent.detailwin is not None:
                        torrent.detailwin.updateSize()

        if self.displayunitrate.GetValue():
            if self.abcparams['displayunitrate'] == "0":
                self.abcparams['displayunitrate'] = "1"
                ranktotalspeed = self.parent.guiman.getRankfromID(20)
                if ranktotalspeed != -1:
                    totalspeedunit = ' ' + self.utility.kbps
                    for index in xrange(len(self.queue.proctab)):
                        torrent = self.queue.proctab[index]
                        if torrent.status == 'pause' or torrent.status == 'onhold':
                            self.list.SetStringItem(index, ranktotalspeed, self.list.GetItem(index, ranktotalspeed).GetText() + totalspeedunit)
        elif self.abcparams['displayunitrate'] == "1":
            self.abcparams['displayunitrate'] = "0"
            ranktotalspeed = self.parent.guiman.getRankfromID(20)
            if ranktotalspeed != -1:
                totalspeedunitlength = len(self.utility.kbps) + 1
                for index in xrange(len(self.queue.proctab)):
                    torrent = self.queue.proctab[index]
                    if torrent.status == 'pause' or torrent.status == 'onhold':
                        self.list.SetStringItem(index, ranktotalspeed, self.list.GetItem(index, ranktotalspeed).GetText()[:-totalspeedunitlength])

        self.abcparams['scrolllistud'] = str(int(self.scrolllistud.GetValue()))

        self.abcparams['scrolllisttb'] = str(int(self.scrolllisttb.GetValue()))

        self.abcparams['rubmessages'] = str(int(self.rubmessages.GetValue()))

        self.abcparams['rubmsgbutton'] = str(self.rubmsgbutton.GetSelection())

        if self.msgindetails.GetValue():
            if self.abcparams['msgindetails'] == "0":
                self.abcparams['msgindetails'] = "1"
                # Update details win if open
                for torrent in self.queue.proctab:
                    if torrent.infowin is not None and torrent.message:
                        torrent.infowin.showMessage()
        elif self.abcparams['msgindetails'] == "1":
            self.abcparams['msgindetails'] = "0"
            # Update details win if open
            for torrent in self.queue.proctab:
                if torrent.infowin is not None:
                    torrent.infowin.hideMessage()

        self.abcparams['keeptermevtmsg'] = str(int(self.keeptermevtmsg.GetValue()))

        self.abcparams['erasetrackal'] = str(int(self.erasetrackal.GetValue()))

        self.abcparams['displayscrapingmsg'] = str(int(self.displayscrapingmsg.GetValue()))

        self.abcparams['displayscrapestatus'] = str(int(self.displayscrapestatus.GetValue()))

        self.abcparams['displaycheckstatus'] = str(int(self.displaycheckstatus.GetValue()))

        self.abcparams['displaytimeout'] = str(int(self.displaytimeout.GetValue()))

        self.abcparams['mousemode'] = str(int(self.mousemode.GetValue()))

        self.abcparams['scrollbelowfinished'] = str(int(self.scrollbelowfinished.GetValue()))

        self.abcparams['countonhold'] = str(int(self.countonhold.GetValue()))

        if self.showschedcounters.GetValue():
            if self.abcparams['showschedcounters'] == "0":
                self.abcparams['showschedcounters'] = "1"
                self.parent.abc_sb.showCell(7)
        elif self.abcparams['showschedcounters'] == "1":
            self.abcparams['showschedcounters'] = "0"
            self.parent.abc_sb.hideCell(7)

        self.abcparams['countonlyalive'] = str(int(self.countonlyalive.GetValue()))

        self.abcparams['showactivitymax'] = str(int(self.showactivitymax.GetValue()))

        self.abcparams['displaydownvol'] = str(int(self.displaydownvol.GetValue()))

        self.abcparams['displayupvol'] = str(int(self.displayupvol.GetValue()))

        self.abcparams['savesizesonexit'] = str(int(self.savesizesonexit.GetValue()))

        self.abcparams['confirmonclose'] = str(int(self.confirmonclose.GetValue()))

        torwaitingforleechers = self.queue.torrentwaitingforleechers
        if self.trig_finish_seed.GetSelection() == 1:
            if self.abcparams['trigwhenfinishseed'] == "0":
                self.queue.currentproc += self.queue.torrentseeding
                if torwaitingforleechers and torwaitingforleechers.abcengine and torwaitingforleechers.status == 'completed':
                    self.queue.currentproc -= 1
                self.abcparams['trigwhenfinishseed'] = "1"
        elif self.abcparams['trigwhenfinishseed'] == "1":
            self.queue.currentproc -= self.queue.torrentseeding
            if torwaitingforleechers and torwaitingforleechers.abcengine and torwaitingforleechers.status == 'completed':
                self.queue.currentproc += 1
            self.abcparams['trigwhenfinishseed'] = "0"

        needslayout = False

        movetopbtnconf = self.movemidbtn.Get3StateValue()
        if movetopbtnconf != int(self.abcparams['movemidbtn']):
            self.abcparams['movemidbtn'] = str(movetopbtnconf)
            if movetopbtnconf == 0:
                self.parent.window.buttontop.Show()
                self.parent.window.buttonmid.Hide()
            elif movetopbtnconf == 1:
                self.parent.window.buttontop.Hide()
                self.parent.window.buttonmid.Show()
            else:
                self.parent.window.buttontop.Show()
                self.parent.window.buttonmid.Show()
            needslayout = True

        if self.stopresallbtn.GetValue():
            if self.abcparams['stopresallbtn'] == "0":
                self.abcparams['stopresallbtn'] = "1"
                self.parent.window.stopallbutton.Show()
                self.parent.window.resumeallbutton.Show()
                needslayout = True
        elif self.abcparams['stopresallbtn'] == "1":
            self.abcparams['stopresallbtn'] = "0"
            self.parent.window.stopallbutton.Hide()
            self.parent.window.resumeallbutton.Hide()
            needslayout = True

        if self.stoptrafficbtn.GetValue():
            if self.abcparams['stoptrafficbtn'] == "0":
                self.abcparams['stoptrafficbtn'] = "1"
                self.parent.window.stoptrafficbutton.Show()
                needslayout = True
        elif self.abcparams['stoptrafficbtn'] == "1":
            self.abcparams['stoptrafficbtn'] = "0"
            self.parent.window.stoptrafficbutton.Hide()
            needslayout = True

        if self.schedulerbtn.GetValue():
            if self.abcparams['schedulerbtn'] == "0":
                self.abcparams['schedulerbtn'] = "1"
                self.parent.window.schedulerbutton.Show()
                needslayout = True
        elif self.abcparams['schedulerbtn'] == "1":
            self.abcparams['schedulerbtn'] = "0"
            self.parent.window.schedulerbutton.Hide()
            needslayout = True

        if self.scannerbtn.GetValue():
            if self.abcparams['scannerbtn'] == "0":
                self.abcparams['scannerbtn'] = "1"
                self.parent.window.scannerbutton.Show()
                needslayout = True
        elif self.abcparams['scannerbtn'] == "1":
            self.abcparams['scannerbtn'] = "0"
            self.parent.window.scannerbutton.Hide()
            needslayout = True

        if self.webservbtn.GetValue():
            if self.abcparams['webservbtn'] == "0":
                self.abcparams['webservbtn'] = "1"
                self.parent.window.webservicebutton.Show()
                needslayout = True
        elif self.abcparams['webservbtn'] == "1":
            self.abcparams['webservbtn'] = "0"
            self.parent.window.webservicebutton.Hide()
            needslayout = True

        needsrestyle = False
        if self.flatbotbtn.GetValue():
            if self.abcparams['flatbotbtn'] == "0":
                self.abcparams['flatbotbtn'] = "1"
                needsrestyle = True
                flat = True
        elif self.abcparams['flatbotbtn'] == "1":
            self.abcparams['flatbotbtn'] = "0"
            needsrestyle = True
            flat = False
        if needsrestyle:
            self.parent.window.buttonup.setFlat(flat)
            self.parent.window.buttondown.setFlat(flat)
            self.parent.window.buttontop.setFlat(flat)
            self.parent.window.buttonmid.setFlat(flat)
            self.parent.window.buttonbottom.setFlat(flat)
            self.parent.window.modebutton.setFlat(flat)
            self.parent.window.stoptrafficbutton.setFlat(flat)
            self.parent.window.stopallbutton.setFlat(flat)
            self.parent.window.resumeallbutton.setFlat(flat)
            self.parent.window.pauseallbutton.setFlat(flat)
            self.parent.window.clearcompletedbutton.setFlat(flat)
            self.parent.window.schedulerbutton.setFlat(flat)
            self.parent.window.scannerbutton.setFlat(flat)
            self.parent.window.webservicebutton.setFlat(flat)

        if needslayout:
            if self.abcparams['schedulerbtn'] == "0" and self.abcparams['scannerbtn'] == "0" \
               and self.abcparams['webservbtn'] == "0":
                self.parent.window.bottomlinespace.Hide()
            else:
                self.parent.window.bottomlinespace.Show()
            self.parent.window.bottomline.Layout()
            self.parent.window.bottompanel.Fit()

        if needsrestyle or needslayout:
            self.parent.window.bottompanel.Refresh()

        newutpex = self.utpex.GetValue()
        if newutpex != self.abcparams['utpex']:
            self.abcparams['utpex'] = str(int(newutpex))
            for tor in self.queue.proctab:
                if tor.abcengine and tor.abcengine.dow:
                    tor.abcengine.dow.setPex(newutpex)

        self.abcparams['dht'] = str(int(self.dht.GetValue()))
        self.abcparams['dhtport'] = str(dhtport)

        if self.utility.hasipv6:
            self.abcparams['ipv6'] = str(int(self.ipv6.GetValue()))
        else:
            self.abcparams['ipv6'] = "0"

        self.abcparams['kickban'] = str(int(self.kickban.GetValue()))

        self.abcparams['notsameip'] = str(int(self.notsameip.GetValue()))

        self.abcparams['scrape'] = str(int(self.scrape.GetValue()))

        self.abcparams['starthome'] = str(int(self.starthome.GetValue()))

        self.abcparams['homepage'] = self.homepage.GetValue()

        if self.stripedlist.GetValue():
            if self.abcparams['stripedlist'] == "0":
                self.abcparams['stripedlist'] = "1"
                self.queue.updateStripesColour()
        else:
            if self.abcparams['stripedlist'] == "1":
                self.abcparams['stripedlist'] = "0"
                self.queue.updateStripesColour(clear = True)

        # Torrent and stripes colours
        col = self.colok.GetBackgroundColour()
        self.abcparams['colok'] = '%d,%d,%d' % (col.Red(), col.Green(), col.Blue())
        col = self.colnoinc.GetBackgroundColour()
        self.abcparams['colnoinc'] = '%d,%d,%d' % (col.Red(), col.Green(), col.Blue())
        col = self.colnocom.GetBackgroundColour()
        self.abcparams['colnocom'] = '%d,%d,%d' % (col.Red(), col.Green(), col.Blue())
        col = self.colnocon.GetBackgroundColour()
        self.abcparams['colnocon'] = '%d,%d,%d' % (col.Red(), col.Green(), col.Blue())
        col = self.coldisc.GetBackgroundColour()
        self.abcparams['coldisc'] = '%d,%d,%d' % (col.Red(), col.Green(), col.Blue())
        col = self.colstripes.GetBackgroundColour()
        self.abcparams['colstripes'] = '%d,%d,%d' % (col.Red(), col.Green(), col.Blue())

        # List font
        self.abcparams['listfont'] = self.listfontface
        self.abcparams['listfontsize'] = str(self.listfontsize)
        self.abcparams['listfontstyle'] = str(self.listfontstyle)
        self.abcparams['listfontweight'] = str(self.listfontweight)

        # Saving value to abc.conf
        tcfm = TorrentConfigFileManager(path.join(self.utility.datapath, "abc.conf"), self.abcparams)
        tcfm.writeAllConfig()

        self.queue.updateRunningTorrentCounters()

        if not self.parent.window.torrentlistloading:
            self.queue.invokeLater(self.queue.scheduler)

        if self.advancedMenuBox is not None:
            self.advancedMenuBox.onClose()

        return True

    def onOK(self, event):
        if self.onApply(event):
            self.EndModal(wx.ID_OK)


class SetVolCheckDateDialog(wx.Dialog):
    def __init__(self, parent, ID, datetime, pos):
        pre = wx.PreDialog()
        pre.Create(parent, ID, '', pos, (-1, -1), wx.DEFAULT_DIALOG_STYLE & ~ wx.CAPTION)
        self.this = pre.this
        self.utility = parent.utility
        self.localize = self.utility.lang.get
        self.datetime = datetime

        p = wx.Panel(self, -1)
        outerbox = wx.BoxSizer(wx.VERTICAL)
        vertsizer = wx.BoxSizer(wx.VERTICAL)
        horsizer = wx.BoxSizer(wx.HORIZONTAL)

        setdatetimebox = wx.StaticBoxSizer(wx.StaticBox(p, -1, ''), wx.HORIZONTAL)

        self.dp = wx.DatePickerCtrl(p, size = (80, -1), dt = datetime, style = wx.DP_DROPDOWN | wx.DP_SHOWCENTURY)

        horsizer.Add(self.dp)

        self.tc = masked.TimeCtrl(p, -1, fmt24hr = True, display_seconds = False)
        #self.tc.SetFont(parent.commandlist.GetFont())

        h = self.tc.GetSize().height
        spin = wx.SpinButton(p, -1, (-1, -1), (h * 2 / 3, h), wx.SP_VERTICAL)
        self.tc.BindSpinButton(spin)
        self.tc.SetValue(datetime)

        horsizer.Add(self.tc, 0, wx.LEFT, 10)
        horsizer.Add(spin, 0, wx.ALIGN_CENTRE_VERTICAL)

        setdatetimebox.Add(horsizer, 0, wx.TOP, 2)

        vertsizer.Add(setdatetimebox)

        buttonbox = wx.BoxSizer(wx.HORIZONTAL)
        self.okbtn = wx.Button(p, -1, self.localize('ok'), style = wx.BU_EXACTFIT)
        deffont = self.okbtn.GetFont()
        deffont.SetPointSize(7)
        self.okbtn.SetFont(deffont)
        self.okbtn.SetDefault()
        self.cancelbtn = wx.Button(p, -1, self.localize('cancel'), style = wx.BU_EXACTFIT)
        self.cancelbtn.SetFont(deffont)
        self.Bind(wx.EVT_BUTTON, self.onOK, self.okbtn)
        self.Bind(wx.EVT_BUTTON, self.onCancel, self.cancelbtn)
        buttonbox.Add(self.okbtn)
        buttonbox.Add(self.cancelbtn, 0, wx.LEFT, 5)

        vertsizer.Add(buttonbox, 0, wx.ALIGN_CENTER_HORIZONTAL | wx.TOP | wx.BOTTOM, 5)

        outerbox.Add(vertsizer, 0, wx.LEFT | wx.RIGHT, 5)

        p.SetAutoLayout(True)
        p.SetSizer(outerbox)
        p.Fit()
        self.SetClientSize(p.GetSize())

    def onOK(self, event):
        self.datetime = self.dp.GetValue()
        time = self.tc.GetWxDateTime()
        self.datetime.SetHour(time.GetHour())
        self.datetime.SetMinute(time.GetMinute())
        self.datetime.SetSecond(time.GetSecond())
        self.EndModal(wx.ID_OK)

    def onCancel(self, event):
        self.EndModal(wx.ID_CANCEL)


class ABCAdvancedOptionWindow(wx.Frame):
    def __init__(self, parent):
        self.parent = parent
        self.utility = self.parent.utility
        self.localize = self.utility.lang.get
        self.abcparams = self.utility.abcparams
        self.queue = self.parent.queue
        self.FONT = 8

        wx.Frame.__init__(self, None, -1, self.localize('abcadvancepref'), style = wx.DEFAULT_DIALOG_STYLE)
        if sys.platform == 'win32':
            self.SetIcon(self.parent.parent.icon)

        self.panel = wx.Panel(self, -1)

        outerbox = wx.BoxSizer(wx.VERTICAL)

        advsettingbox = wx.StaticBoxSizer(wx.StaticBox(self.panel, -1, ''), wx.VERTICAL)

        colsizer = wx.FlexGridSizer(cols = 1, hgap = 13, vgap = 10)

        self.ip_data = wx.TextCtrl(parent = self.panel, id = -1, value = self.abcparams['ip'],
                                   size = (self.FONT * 13, -1))

        self.bind_data = wx.TextCtrl(parent = self.panel, id = -1, value = self.abcparams['bind'],
                                     size = (self.FONT * 13, -1))

        if self.utility.hasipv6:
            self.ipv6bindsv4_data = wx.Choice(self.panel, -1, choices = [self.localize('separatesockets'), self.localize('singlesocket')])
            self.ipv6bindsv4_data.SetSelection(int(self.abcparams['ipv6_binds_v4']))

        self.minpeers_data = SpinBox(self.panel, width = 30, value = str(self.abcparams['min_peers']), min = 10, max = 100)

        self.alloctype_data = wx.Choice(self.panel, -1, choices = [self.localize('normalalloc'), self.localize('background'),
                                                              self.localize('prealloc'), self.localize('sparse')])
        self.alloctype_data.SetSelection(int(self.abcparams['alloc_type']))

        allocratebox = wx.BoxSizer(wx.HORIZONTAL)
        self.allocrate_data = SpinBox(self.panel, width = 30, value = str(self.abcparams['alloc_rate']), min = 1, max = 100)
        allocratebox.Add(self.allocrate_data.getSizer(), 0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        allocratebox.Add(self.staticText(self.utility.mbps), 0, wx.ALIGN_CENTER_VERTICAL)

        self.locking_data = wx.Choice(self.panel, -1, choices = [self.localize('locknever'), self.localize('lockwrite'),
                                                       self.localize('lockalways')])
        if int(self.abcparams['lock_files']):
            if int(self.abcparams['lock_while_reading']):
                self.locking_data.SetSelection(2)
            else:
                self.locking_data.SetSelection(1)
        else:
            self.locking_data.SetSelection(0)

        self.doublecheck_data = wx.Choice(self.panel, -1, choices = [self.localize('nocheck'), self.localize('dblcheck'),
                                                                self.localize('trplcheck')])
        if int(self.abcparams['double_check']):
            if int(self.abcparams['triple_check']):
                self.doublecheck_data.SetSelection(2)
            else:
                self.doublecheck_data.SetSelection(1)
        else:
            self.doublecheck_data.SetSelection(0)

        maxfilesopen_choices = ['5', '10', '15', '20', '25', '30', '35', '40', '45', '50', '60', '75', '100', '150', '200', self.localize('nolimit')]
        self.maxfilesopen_data = wx.Choice(self.panel, -1, choices = maxfilesopen_choices)
        setval = int(self.abcparams['max_files_open'])
        if setval == 0:
            setval = self.localize('nolimit')
        else:
            setval = str(setval)
        if not setval in maxfilesopen_choices:
            setval = '20'
        self.maxfilesopen_data.SetStringSelection(setval)

        maxconnections_choices = ['10', '20', '30', '40', '50', '60', '75', '100', '150', '200', self.localize('nolimit')]
        self.maxconnections_data = wx.Choice(self.panel, -1, choices = maxconnections_choices)
        setval = int(self.abcparams['max_connections'])
        if setval == 0:
            setval = self.localize('nolimit')
        else:
            setval = str(setval)
        if not setval in maxconnections_choices:
            setval = '100'
        self.maxconnections_data.SetStringSelection(setval)

        expirecachedata_choices = ['3 ' + self.utility.d, '5 ' + self.utility.d, '7 ' + self.utility.d,
                                   '10 ' + self.utility.d, '15 ' + self.utility.d, '30 ' + self.utility.d,
                                   '60 ' + self.utility.d, '90 ' + self.utility.d, self.localize('never')]
        self.expirecachedata_data = wx.Choice(self.panel, -1, choices = expirecachedata_choices)
        setval = int(self.abcparams['expire_cache_data'])
        if setval == 0:
            setval = self.localize('never')
        else:
            setval = str(setval) + ' ' + self.utility.d
        if not setval in expirecachedata_choices:
            setval = '10 ' + self.utility.d
        self.expirecachedata_data.SetStringSelection(setval)

        writebuffersizebox = wx.BoxSizer(wx.HORIZONTAL)
        self.writebuffersize_data = wx.TextCtrl(parent = self.panel, id = -1, value = self.abcparams['writebuffersize'],
                                                size = (self.FONT * 5, -1))
        self.writebuffersize_data.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        writebuffersizebox.Add(self.writebuffersize_data, 0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        writebuffersizebox.Add(self.staticText(self.utility.mb), 0, wx.ALIGN_CENTER_VERTICAL)

        self.readbuffermemorysaving_data = wx.Choice(self.panel, -1, choices = [self.localize('loweruploads'), self.localize('stopbuffering')])
        self.readbuffermemorysaving_data.SetSelection(int(self.abcparams['readbuffermemorysaving']))

        readbuffermaxglobalsizebox = wx.BoxSizer(wx.HORIZONTAL)
        self.readbuffermaxglobalsize_data = wx.TextCtrl(parent = self.panel, id = -1, value = self.abcparams['readbuffermaxglobalsize'],
                                                   size = (self.FONT * 5, -1))
        self.readbuffermaxglobalsize_data.Bind(wx.EVT_CHAR, self.utility.onWriteDigits)
        self.readbufferglobalsize = self.staticText('%.1f' % (self.queue.readbufferglobalsize / 1048576.), size = (self.FONT * 5, -1))
        readbuffermaxglobalsizebox.Add(self.readbuffermaxglobalsize_data, 0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 5)
        readbuffermaxglobalsizebox.Add(self.staticText(self.utility.mb), 0, wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, 10)
        readbuffermaxglobalsizebox.Add(self.staticText(self.localize('current')), 0, wx.ALIGN_CENTER_VERTICAL)
        readbuffermaxglobalsizebox.Add(self.readbufferglobalsize, 0, wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        readbuffermaxglobalsizebox.Add(self.staticText(self.utility.mb), 0, wx.ALIGN_CENTER_VERTICAL)

        twocolsizer = wx.FlexGridSizer(cols = 2, hgap = 20)
        datasizer = wx.FlexGridSizer(cols = 2, hgap = 5, vgap = 2)
        datasizer.Add(self.staticText(self.localize('localip')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(self.ip_data)
        datasizer.Add(self.staticText(self.localize('iptobindto')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(self.bind_data)
        if self.utility.hasipv6:
            datasizer.Add(self.staticText(self.localize('ipv6sockethandling')), 1, wx.ALIGN_CENTER_VERTICAL)
            datasizer.Add(self.ipv6bindsv4_data)
        datasizer.Add(self.staticText(self.localize('minnumberofpeer')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(self.minpeers_data.getSizer())
        datasizer.Add(self.staticText(self.localize('diskalloctype')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(self.alloctype_data)
        datasizer.Add(self.staticText(self.localize('allocrate')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(allocratebox)
        datasizer.Add(self.staticText(self.localize('filelocking')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(self.locking_data)
        datasizer.Add(self.staticText(self.localize('extradatachecking')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(self.doublecheck_data)
        datasizer.Add(self.staticText(self.localize('maxfileopen')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(self.maxfilesopen_data)
        datasizer.Add(self.staticText(self.localize('maxpeerconnection')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(self.maxconnections_data)
        datasizer.Add(self.staticText(self.localize('expirecachedata')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(self.expirecachedata_data)
        datasizer.Add(self.staticText(self.localize('writebuffersize')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(writebuffersizebox)
        datasizer.Add(self.staticText(self.localize('readbuffermemorysaving')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(self.readbuffermemorysaving_data)
        datasizer.Add(self.staticText(self.localize('readbuffermaxglobalsize')), 1, wx.ALIGN_CENTER_VERTICAL)
        datasizer.Add(readbuffermaxglobalsizebox)

        twocolsizer.Add(datasizer)

        infosizer = wx.FlexGridSizer(cols = 1)
        self.hinttext = self.staticText('', self.FONT, False, 'Blue')
        infosizer.Add(self.hinttext, 1, wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)
        infosizer.SetMinSize((180, 100))
        twocolsizer.Add(infosizer, 1, wx.EXPAND)

        colsizer.Add(twocolsizer)

        colsizer.Add(self.staticText(self.localize('advsettingwarning1'), self.FONT - 1), 1, wx.ALIGN_CENTER)
        colsizer.Add(self.staticText(self.localize('advsettingwarning2'), self.FONT - 1), 1, wx.ALIGN_CENTER)

        advsettingbox.Add(colsizer, 1, wx.EXPAND)
        outerbox.Add(advsettingbox, 0, wx.EXPAND | wx.TOP, -4)

        button_box = wx.BoxSizer(wx.HORIZONTAL)

        defaultsbutton = wx.Button(self.panel, -1, self.localize('reverttodefault'))
        button_box.Add(defaultsbutton, 0, wx.ALL, 5)

        applybutton = wx.Button(self.panel, -1, self.localize('apply'), size = (60, -1))
        button_box.Add(applybutton, 0, wx.ALL, 5)

        okbutton = wx.Button(self.panel, -1, self.localize('ok'), size = (60, -1))
        button_box.Add(okbutton, 0, wx.ALL, 5)

        cancelbutton = wx.Button(self.panel, -1, self.localize('cancel'), size = (60, -1))
        button_box.Add(cancelbutton, 0, wx.ALL, 5)

        outerbox.Add(button_box, 0, wx.ALIGN_CENTER)

        self.panel.SetAutoLayout(True)
        self.panel.SetSizer(outerbox)
        self.panel.Fit()
        self.SetClientSize(self.panel.GetSize())

        wx.EVT_BUTTON(self, applybutton.GetId(), self.onApply)
        wx.EVT_BUTTON(self, okbutton.GetId(), self.onOK)
        wx.EVT_BUTTON(self, cancelbutton.GetId(), self.onCancel)
        wx.EVT_BUTTON(self, defaultsbutton.GetId(), self.setDefaults)        
        wx.EVT_CLOSE(self, self.onClose)
        wx.EVT_ENTER_WINDOW(self.ip_data, self.ip_hint)
        wx.EVT_ENTER_WINDOW(self.bind_data, self.bind_hint)
        if self.utility.hasipv6:
            wx.EVT_ENTER_WINDOW(self.ipv6bindsv4_data, self.ipv6bindsv4_hint)
        wx.EVT_ENTER_WINDOW(self.minpeers_data.getTextWindow(), self.minpeers_hint)
        wx.EVT_ENTER_WINDOW(self.minpeers_data.getSpinWindow(), self.minpeers_hint)
        wx.EVT_ENTER_WINDOW(self.alloctype_data, self.alloctype_hint)
        wx.EVT_ENTER_WINDOW(self.allocrate_data.getTextWindow(), self.allocrate_hint)
        wx.EVT_ENTER_WINDOW(self.allocrate_data.getSpinWindow(), self.allocrate_hint)
        wx.EVT_ENTER_WINDOW(self.locking_data, self.locking_hint)
        wx.EVT_ENTER_WINDOW(self.doublecheck_data, self.doublecheck_hint)        
        wx.EVT_ENTER_WINDOW(self.maxfilesopen_data, self.maxfilesopen_hint)
        wx.EVT_ENTER_WINDOW(self.maxconnections_data, self.maxconnections_hint)
        wx.EVT_ENTER_WINDOW(self.expirecachedata_data, self.expirecachedata_hint)
        wx.EVT_ENTER_WINDOW(self.writebuffersize_data, self.writebuffersize_hint)
        wx.EVT_ENTER_WINDOW(self.readbuffermemorysaving_data, self.readbuffermemorysaving_hint)
        wx.EVT_ENTER_WINDOW(self.readbuffermaxglobalsize_data, self.readbuffermaxglobalsize_hint)

        xoptions, yoptions = self.parent.GetPosition()
        xadv = xoptions + (self.parent.GetSize()[0] - self.GetSize()[0]) / 2
        yadv = yoptions + 22
        if xadv < 0:
            xadv = 0
        if yadv < 0:
            yadv = 0
        self.SetPosition((xadv, yadv))
        self.Show()

    def staticText(self, text, font = None, underline = False, color = None, size = (-1, -1)):
        if font is None:
            font = self.FONT
        x = wx.StaticText(self.panel, -1, text, style = wx.ALIGN_LEFT, size = size)
        x.SetFont(wx.Font(font, wx.DEFAULT, wx.NORMAL, wx.NORMAL, underline))
        if color is not None:
            x.SetForegroundColour(color)
        return x

    def setDefaults(self, event):
        self.ip_data.SetValue('')
        self.bind_data.SetValue('')
        if self.utility.hasipv6:
            self.ipv6bindsv4_data.SetSelection(1)
        self.minpeers_data.setValue(30)
        self.alloctype_data.SetStringSelection(self.localize('normalalloc'))
        self.allocrate_data.setValue(2)
        self.locking_data.SetSelection(1)
        self.doublecheck_data.SetSelection(1)
        self.maxfilesopen_data.SetStringSelection('10')
        self.maxconnections_data.SetStringSelection('40')
        self.expirecachedata_data.SetStringSelection('15 ' + self.utility.d)
        self.writebuffersize_data.SetValue('2')
        self.readbuffermemorysaving_data.SetStringSelection(self.localize('loweruploads'))
        self.readbuffermaxglobalsize_data.SetValue('64')

    def onApply(self, event):
        self.abcparams['ip'] = self.ip_data.GetValue()
        self.abcparams['bind'] = self.bind_data.GetValue()
        if self.utility.hasipv6:
            self.abcparams['ipv6_binds_v4'] = str(self.ipv6bindsv4_data.GetSelection())
        self.abcparams['min_peers'] = self.minpeers_data.getValue()
        self.abcparams['alloc_type'] = str(self.alloctype_data.GetSelection())
        self.abcparams['alloc_rate'] = self.allocrate_data.getValue()
        self.abcparams['lock_files'] = str(int(self.locking_data.GetSelection() >= 1))
        self.abcparams['lock_while_reading'] = str(int(self.locking_data.GetSelection() > 1))
        self.abcparams['double_check'] = str(int(self.doublecheck_data.GetSelection() >= 1))
        self.abcparams['triple_check'] = str(int(self.doublecheck_data.GetSelection() > 1))

        try:
            max_files_open = int(self.maxfilesopen_data.GetStringSelection())
        except:       # If it ain't a number, it must be "no limit"
            max_files_open = 0
        self.abcparams['max_files_open'] = str(max_files_open)

        try:
            max_connections = int(self.maxconnections_data.GetStringSelection())
            max_initiate = min(2 * int(self.abcparams['min_peers']), max_connections)
        except:       # If it ain't a number, it must be "no limit"
            max_connections = 0
            max_initiate = 2 * int(self.abcparams['min_peers'])
        self.abcparams['max_connections'] = str(max_connections)
        self.abcparams['max_initiate'] = str(max_initiate)

        try:
            expire_cache_data = int(self.expirecachedata_data.GetStringSelection()[:-len(self.utility.d) - 1])
        except:       # If it ain't a number, it must be "never"
            expire_cache_data = 0
        self.abcparams['expire_cache_data'] = str(expire_cache_data)

        try:
            writebuffersize_int = int(self.writebuffersize_data.GetValue())
        except:
            newwritebuffersize = '2'
        else:
            newwritebuffersize = str(writebuffersize_int)
        if newwritebuffersize != self.abcparams['writebuffersize']:
            self.abcparams['writebuffersize'] = newwritebuffersize
            newwritebuffersize_int = int(newwritebuffersize)
            for tor in self.queue.proctab:
                if tor.abcengine and tor.abcengine.dow:
                    tor.abcengine.dow.setWriteBufferSize(newwritebuffersize_int)

        newreadbuffermemorysaving = str(self.readbuffermemorysaving_data.GetSelection())
        if newreadbuffermemorysaving != self.abcparams['readbuffermemorysaving']:
            self.abcparams['readbuffermemorysaving'] = newreadbuffermemorysaving
            for tor in self.queue.proctab:
                if tor.abcengine and tor.abcengine.dow:
                    tor.abcengine.dow.setReadBufferMemorySaving(int(newreadbuffermemorysaving))

        try:
            readbuffermaxglobalsize_int = int(self.readbuffermaxglobalsize_data.GetValue())
        except:
            newreadbuffermaxglobalsize = '100'
        else:
            newreadbuffermaxglobalsize = str(readbuffermaxglobalsize_int)
        if newreadbuffermaxglobalsize != self.abcparams['readbuffermaxglobalsize']:
            self.abcparams['readbuffermaxglobalsize'] = newreadbuffermaxglobalsize
            self.queue.readbuffermaxglobalsize = int(newreadbuffermaxglobalsize) * 1048576L

        # Saving value to abc.conf
        tcfm = TorrentConfigFileManager(path.join(self.utility.datapath, "abc.conf"), self.abcparams)
        tcfm.writeAllConfig()

        return True

    def onOK(self, event):
        if self.onApply(event):
            self.onClose()

    def onCancel(self, event):
        self.onClose()

    def onClose(self, event = None):
        self.parent.advancedMenuBox = None
        self.Destroy()

    def ip_hint(self, event):
        self.hinttext.SetLabel(self.localize('iphint'))

    def bind_hint(self, event):
        self.hinttext.SetLabel(self.localize('bindhint'))

    def ipv6bindsv4_hint(self, event):
        self.hinttext.SetLabel(self.localize('ipv6bindsv4hint'))

    def minpeers_hint(self, event):
        self.hinttext.SetLabel(self.localize('minpeershint'))

    def alloctype_hint(self, event):
        self.hinttext.SetLabel(self.localize('alloctypehint'))

    def allocrate_hint(self, event):
        self.hinttext.SetLabel(self.localize('allocratehint'))

    def locking_hint(self, event):
        self.hinttext.SetLabel(self.localize('lockinghint'))

    def doublecheck_hint(self, event):
        self.hinttext.SetLabel(self.localize('doublecheckhint'))

    def maxfilesopen_hint(self, event):
        self.hinttext.SetLabel(self.localize('maxfilesopenhint'))

    def maxconnections_hint(self, event):
        self.hinttext.SetLabel(self.localize('maxconnectionshint'))

    def expirecachedata_hint(self, event):
        self.hinttext.SetLabel(self.localize('expirecachedatahint'))

    def writebuffersize_hint(self, event):
        self.hinttext.SetLabel(self.localize('writebuffersizehint'))

    def readbuffermemorysaving_hint(self, event):
        self.hinttext.SetLabel(self.localize('readbuffermemorysavinghint'))

    def readbuffermaxglobalsize_hint(self, event):
        self.hinttext.SetLabel(self.localize('readbuffermaxglobalsizehint'))
